/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 TCP client for time service

 Build: a_tcptime_cli
*/
#include "../src/wysockfile.h"
#include "../src/wysockaddrinet4.h"
#include "../src/wydate.h"
#include "../src/wy_uty.h"

const time_t UNIX_EPOCH= 2208988800UL;

static const char* syntax=
 "[Syntax]$./a_tcptime_cli ip port\n"
 " ip  = server ip in dotted-decimal notation\n"
 " port= port number\n";

int main(int argc, char* argv[])
try {
 WyRet r;
 uint16_t port;
 const char* SvrIP;

 if(argc==3) {
   SvrIP=argv[1];
   if((r=Wy::_strnum(port,NULL,argv[2]))!=OK) {
     WY_THROW(r);
   }
 } else {
   if(argc!=1) {
     Wy::cout << syntax;
     return(-1);
   }
   port=37;
   SvrIP="127.0.0.1";
 }

 const WySockAddrINet4 SvrAddr(SvrIP,port);
 WySockFile sock(PF_INET,SOCK_STREAM,0);
 const char TrigMsg[]="Request for time\n";
 size_t tlen;
 if((r=sock.connect(SvrAddr))!=OK) {
   WY_THROW(r);
 }
 sock << TrigMsg;

 time_t tm;
 tlen=sizeof(tm);
 for(char* cptr=(char*)&tm ; tlen>0; ) {
   if((r=sock.read(cptr,tlen,tlen))!=OK) {
     WY_THROW(r);
   }
   cptr+=tlen;
   tlen=sizeof(tm)-tlen;
 }
 tm=ntohl(tm)-UNIX_EPOCH;

 const WyDate date(Wy::local_tzname(), WyTimeSpec(tm,0));
 
 Wy::cout << "Time from "
          << "ip=" << Wy::wrd(SvrAddr.ip())
          << " port=" << SvrAddr.port() << '\n'
          << Wy::wrd(date) << "\n\n";
 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
