/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 TCP server for time service

 Build: make a_tcptime_svr
*/
#include "../src/wysockfile.h"
#include "../src/wysockaddrinet4.h"
#include "../src/wydate.h"
#include "../src/wy_uty.h"

const time_t UNIX_EPOCH= 2208988800UL;

static WyRet time_service_for(WySockFile& cli_sock, const WySockAddrINet4& cli_addr) throw()
try {
 WyRet r;
 char rbuf[256];
 size_t rlen;
 if((r=cli_sock.read(rbuf,sizeof(rbuf)-1,rlen))!=OK) {
   WY_THROW(r);
 }
 rbuf[rlen]=0;

 Wy::cout << "Connection from " << Wy::wrd(cli_addr.ip())
          << " (" << cli_addr.port() << "): "
          << rbuf << '\n';

 const WyTimeSpec tm_now( Wy::now() );
 time_t tm(tm_now.second()+UNIX_EPOCH);
 tm=htonl(tm);
 if((r=cli_sock.write(&tm,sizeof(tm),rlen))!=OK) {
   Wy::cerr << Wy::wrd(r) << '\n';
 }
 if(rlen!=sizeof(tm)) {
   Wy::cerr << "written bytes is not " << sizeof(tm) << "\n";
 }
 return(OK);
}
catch(const WyByteFlow::Reply& e) {
  WY_RETURN(e);
}
catch(...) {
 WY_TERMINATE("unknown throw type");
};

static const char* syntax=
 "[Syntax]$./a_tcptime_svr port\n";

int main(int argc, char* argv[])
try {
 if(argc!=2) {
   Wy::cout << syntax;
   return(-1);
 }
 WyRet r;
 uint16_t port;

 if((r=Wy::_strnum(port,NULL,argv[1]))!=OK) {
   WY_THROW(r);
 }

 WySockFile svr_sock(PF_INET,SOCK_STREAM,0);
 const WySockAddrINet4 SvrAddr(INADDR_ANY,port);

 if((r=svr_sock.bind(SvrAddr))!=OK) {
   WY_THROW(r);
 }
 if((r=svr_sock.listen(0))!=OK) {
   WY_THROW(r);
 }

 for(;;) {
   WySockFile a_sock;
   WySockAddrINet4 a_addr;

   if((r=svr_sock.accept(a_sock,a_addr))!=OK) {
     WY_THROW(r);
   }
   if((r=time_service_for(a_sock,a_addr))!=OK) {
     Wy::cerr << " .. Service failed\n";
     // RESUME
   }
 }
 // UNREACHABLE
 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
