/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 UDP server for time service

 Build: make a_udptime_svr
*/
#include "../src/wysockfile.h"
#include "../src/wysockaddrinet4.h"
#include "../src/wydate.h"
#include "../src/wy_uty.h"

const time_t UNIX_EPOCH= 2208988800UL;

static const char* syntax=
 "[Syntax]$./a_udptime_svr port\n";

int main(int argc, char* argv[])
try {
 if(argc!=2) {
   Wy::cout << syntax;
   return(-1);
 }
 WyRet r;
 uint16_t port;

 if((r=Wy::_strnum(port,NULL,argv[1]))!=OK) {
   WY_THROW(r);
 }

 WySockFile svr_sock(PF_INET,SOCK_DGRAM,0);
 const WySockAddrINet4 SvrAddr(INADDR_ANY,port);

 if((r=svr_sock.bind(SvrAddr))!=OK) {
   WY_THROW(r);
 }

 for(;;) {
   WySockAddrINet4 taddr;
   char rbuf[256];
   size_t rlen;

   if((r=svr_sock.recvfrom(rbuf,sizeof(rbuf)-1,0,&taddr,rlen))!=OK) {
     WY_THROW(r);
   }
   rbuf[rlen]=0;

   Wy::cout << "Connection from " << Wy::wrd(taddr.ip())
            << " (" << taddr.port() << "): "
            << rbuf << '\n';

   const WyTimeSpec tm_now( Wy::now() );
   time_t tm(tm_now.second()+UNIX_EPOCH);
   tm=htonl(tm);
   if((r=svr_sock.sendto(&tm,sizeof(tm),0,&taddr,rlen))!=OK) {
     Wy::cerr << Wy::wrd(r) << '\n';
   }
   if(rlen!=sizeof(tm)) {
     Wy::cerr << "sendto bytes is not " << sizeof(tm) << "\n";
   }
 }
 // UNREACHABLE
 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "unknown throw type\n";
 return(-1);
};
