/* Copyright is licensed under GNU LGPL.                 by IJ. Wang 2006

   Basic Wy_Array chk function template.
   The check with ck_array() is minimal, for it is meant to be common.
*/
#ifndef WY__CK_ARRAY_H__
#define WY__CK_ARRAY_H__

#include "wy_array.h"

template<typename T>
bool wy__ck_array_chk_default(const T& vec)
{
 if(vec.begin()!=vec.end()) {
   return(false);
 }
 if(vec.size()!=0) {
   return(false);
 }
 if(vec.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if(vec.begin()!=vec.end()) {
   WY_THROW( WyRet() );
 }
 if(vec._capacity()<T::min_capacity()) {
   WY_THROW( WyRet() );
 }
 {
  T tmp;
  if(vec!=tmp) {
    WY_THROW( WyRet() );
  }
  if(vec==tmp) {
  } else {
    WY_THROW( WyRet() );
  }
 }
 return(true);
};

#define WY__CK_ARRAY_CHK_DEFAULT(expr) if(wy__ck_array_chk_default(expr)==false) { WY_THROW(WyRet() ); };

template<typename EleType>
void ck_array(void)
{
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  if(tmp.begin()!=tmp.end()) {
    WY_THROW( WyRet() );
  }
  if(tmp._capacity()!=Wy_Array<EleType>::min_capacity()) {
    WY_THROW( WyRet() );
  }
 }

 // chk Wy_Array(const Wy_Array&)
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  Wy_Array<EleType> tmp2(tmp1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);

  tmp2.push_back(EleType());
  Wy_Array<EleType> tmp3(tmp2);
  if(tmp2!=tmp3) {
    WY_THROW( WyRet() );
  }
  if(tmp3.size()!=1) {
    WY_THROW( WyRet() );
  }

  for(int i=0; i<100; ++i) {
    tmp2.push_back(EleType());
  }
  Wy_Array<EleType> tmp4(tmp2);
  if(tmp4!=tmp2) {
    WY_THROW( WyRet() );
  }
  if(tmp4.size()!=tmp2.size()) {
    WY_THROW( WyRet() );
  }
  if(tmp4._capacity()<tmp4.size()) {
    WY_THROW( WyRet() );
  }
 }

 // chk Wy_Array(size_t,const EleType&)
 {
  Wy_Array<EleType> tmp1(40,EleType()),tmp2(tmp1);
  if(tmp1.size()!=40) {
    WY_THROW( WyRet() );
  }
  if(tmp2.size()!=tmp1.size()) {
    WY_THROW( WyRet() );
  }
 }

 // chk reset(void)
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  tmp1.reset();
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  Wy_Array<EleType> tmp2(1,EleType());
  if(tmp2.size()!=1) {
    WY_THROW( WyRet() );
  }
  tmp2.reset();
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);
 }

 // chk reset(const Wy_Array&)
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  Wy_Array<EleType> tmp2;
  tmp2.reset(tmp1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);
  Wy_Array<EleType> tmp3(10,EleType());
  if(tmp3.size()!=10) {
    WY_THROW( WyRet() );
  }
  tmp2.reset(tmp3);
  if(tmp2!=tmp3) {
    WY_THROW( WyRet() );
  }
  tmp2.reset(tmp2);   // self reset
  if(tmp2==tmp3) {
  } else {
    WY_THROW( WyRet() );
  }
 }

 // chk operator=(const Wy_Array&)
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  Wy_Array<EleType> tmp2;
  tmp2=tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);
  Wy_Array<EleType> tmp3(10,EleType());
  if(tmp3.size()!=10) {
    WY_THROW( WyRet() );
  }
  tmp2=tmp3;
  if(tmp2!=tmp3) {
    WY_THROW( WyRet() );
  }
  tmp2=tmp2;   // self assign
  if(tmp2==tmp3) {
  } else {
    WY_THROW( WyRet() );
  }
 }

 // chk swap(Wy_Array&)
 {
  Wy_Array<EleType> tmp1,tmp2;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);
  tmp1.swap(tmp1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  tmp1.swap(tmp2);
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp2);
  Wy_Array<EleType> tmp3(10,EleType());
  tmp1.swap(tmp3);
  if(tmp1.size()!=10) {
    WY_THROW( WyRet() );
  }
  WY__CK_ARRAY_CHK_DEFAULT(tmp3);
 }

 // chk err front()
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);

  try {
    tmp1.front();
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    WY__CK_ARRAY_CHK_DEFAULT(tmp1);
    if(e!=Wym_ENOENT) {
      WY_THROW( WyRet(e) );
    }
  }
  catch(...) {
    WY_THROW( WyRet() );
  };
 }

 // chk err back()
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  try {
    tmp1.back();
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    if(e!=Wym_ENOENT) {
      WY_THROW( WyRet(e) );
    }
  }
  catch(...) {
    WY_THROW( WyRet() );
  };
 }

 // chk err operator[]
 {
  Wy_Array<EleType> tmp1;
  WY__CK_ARRAY_CHK_DEFAULT(tmp1);
  try {
    tmp1[0];
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    WY__CK_ARRAY_CHK_DEFAULT(tmp1);
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet(e) );
    }
  }
  catch(...) {
    WY_THROW( WyRet() );
  };

  tmp1.push_back( EleType() );
  if(tmp1.size()!=1) {
    WY_THROW( WyRet() );
  }
  try {
    tmp1[tmp1.size()];
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet(e) );
    }
    if(tmp1.size()!=1) {
      WY_THROW( WyRet() );
    }
  }
  catch(...) {
    WY_THROW( WyRet() );
  };
 }

 // chk push_back
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  size_t cap( (tmp._capacity()<<1)+1 );
  for(size_t i=0; i<cap; ++i) {
    tmp.push_back( EleType() );
  }
  if(tmp.size()!=cap) {
    WY_THROW( WyRet() );
  }
 }

 // chk pop_back
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.pop_back();
  WY__CK_ARRAY_CHK_DEFAULT(tmp);

  size_t cap( (tmp._capacity()<<1)+1 );
  for(size_t i=0; i<cap; ++i) {
    tmp.push_back(EleType());
  }
  if(tmp.size()!=cap) {
    WY_THROW( WyRet() );
  }
  while(tmp.size()) {
    tmp.pop_back();
  };
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // chk erase
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.erase(0,1);
  WY__CK_ARRAY_CHK_DEFAULT(tmp);

  size_t cap( (tmp._capacity()<<1)+1 );
  for(size_t i=0; i<cap; ++i) {
    tmp.push_back( EleType() );
  }
  if(tmp.size()!=cap) {
    WY_THROW( WyRet() );
  }
  size_t rsize=tmp.size();
  tmp.erase(tmp.size(),100);
  if(tmp.size()!=rsize) {
    WY_THROW( WyRet() );
  }
  tmp.erase(tmp.size()-1,0);
  if(tmp.size()!=rsize) {
    WY_THROW( WyRet() );
  }
  tmp.erase(0,tmp.size());
  if(tmp.size()!=0) {
    WY_THROW( WyRet() );
  }
  rsize=tmp.size();

  try {
    tmp.erase(tmp.size()+1,1);
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet() );
    }
  };
  if(tmp.size()!=rsize) {
    WY_THROW( WyRet() );
  }
 }

 // chk insert
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.insert(0,EleType());
  if(tmp.size()!=1) {
    WY_THROW( WyRet() );
  }
  tmp.insert(0,EleType());
  if(tmp.size()!=2) {
    WY_THROW( WyRet() );
  }
  tmp.insert(0,EleType());
  if(tmp.size()!=3) {
    WY_THROW( WyRet() );
  }
  tmp.insert(1,EleType());
  if(tmp.size()!=4) {
    WY_THROW( WyRet() );
  }
  tmp.insert(tmp.size()-1,EleType());
  if(tmp.size()!=5) {
    WY_THROW( WyRet() );
  }
  tmp.insert(tmp.size(),EleType());
  if(tmp.size()!=6) {
    WY_THROW( WyRet() );
  }

  try {
    tmp.insert(tmp.size()+1,EleType());
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet() );
    }
  };
  if(tmp.size()!=6) {
    WY_THROW( WyRet() );
  }
 }

 // chk replace
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.replace(0,1,0,EleType());
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.replace(0,1,2,EleType());
  if(tmp.size()!=2) {
    WY_THROW( WyRet() );
  }
  tmp.replace(1,1,2,EleType());
  if(tmp.size()!=3) {
    WY_THROW( WyRet() );
  }
  tmp.replace(3,1,2,EleType());
  if(tmp.size()!=5) {
    WY_THROW( WyRet() );
  }
  tmp.replace(1,5,2,EleType());
  if(tmp.size()!=3) {
    WY_THROW( WyRet() );
  }
  tmp.replace(0,5,0,EleType());
  if(tmp.size()!=0) {
    WY_THROW( WyRet() );
  }
 }

 // chk resize
 {
  Wy_Array<EleType> tmp;
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
  tmp.resize(0);
  WY__CK_ARRAY_CHK_DEFAULT(tmp);

  tmp.resize(1);
  if(tmp.size()!=1) {
    WY_THROW( WyRet() );
  }
  tmp.resize(2);
  if(tmp.size()!=2) {
    WY_THROW( WyRet() );
  }
  tmp.resize(100);
  if(tmp.size()!=100) {
    WY_THROW( WyRet() );
  }
  tmp.resize(90);
  if(tmp.size()!=90) {
    WY_THROW( WyRet() );
  }
  try {
    tmp.resize(tmp.max_capacity()+1);
    WY_THROW( WyRet() );
  }
  catch(const typename Wy_Array<EleType>::Reply& e) {
    if(e!=Wym_EFBIG) {
      WY_THROW( WyRet() );
    }
  };
  if(tmp.size()!=90) {
    WY_THROW( WyRet() );
  }
  tmp.resize(1);
  if(tmp.size()!=1) {
    WY_THROW( WyRet() );
  }
  tmp.resize(0);
  if(tmp.size()!=0) {
    WY_THROW( WyRet() );
  }
  WY__CK_ARRAY_CHK_DEFAULT(tmp);
 }

};

#endif
