/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

   Build: ck_num.o
*/

//#include "wystr.h"
#include "wynum.h"
#include <limits>
#include <cstdlib>
#include "wymath.h"

#include <iostream>
#if WYLIB_VERSION!=30
#error Test code is for WYLIB_VERSION 30
#endif

extern void ck_strnum(void);    // defined in ckn_strnum.cpp

template <typename NumType>
static void ck_uty(void)
{
 WyRet r;

 // chk Wy::_strnum(...)
 {
   NumType tnn;
   const char s1[]="52";
   const char* endptr;
   tnn=7;
   if((r=Wy::_strnum(tnn,0,"0",10))!=OK) {
     WY_THROW(r);
   }
   if(tnn!=NumType(0)) {
     WY_THROW(r);
   }
   if((r=Wy::_strnum(tnn,NULL,WyCSeg("0",1),10))!=OK) {
     WY_THROW(r);
   }
   if(tnn!=NumType(0)) {
     WY_THROW(r);
   }

   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s1,sizeof(s1)-1),16))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s1!=sizeof(s1)-1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0x52)) {
     WY_THROW( WyRet() );
   }

   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s1,sizeof(s1)-1),0))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s1!=sizeof(s1)-1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(52)) {
     WY_THROW( WyRet() );
   }

   const char s2[]="0x041";
   int radix;
   radix=0;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s2,sizeof(s2)-1),radix))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s2!=sizeof(s2)-1) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0x41)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s2+2,sizeof(s2)-3),radix))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s2!=sizeof(s2)-1) {
     WY_THROW( WyRet() );
   }
   if(radix!=8) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(041)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s2,1),radix))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s2!=1) {
     WY_THROW( WyRet() );
   }
   if(radix!=10) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg("00"),radix))!=OK) {
     WY_THROW(r);
   }
   if(radix!=8) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0)) {
     WY_THROW( WyRet() );
   }

   radix=5;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(s2+2,sizeof(s2)-3),radix))!=OK) {
     WY_THROW(r);
   }
   if(endptr-s2!=sizeof(s2)-1) {
     WY_THROW( WyRet() );
   }
   if(radix!=5) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(21)) {
     WY_THROW( WyRet() );
   }
 
   if(std::numeric_limits<NumType>::is_signed) {
     const char ns1[]="-52";
     if((r=Wy::_strnum(tnn,0,WyCSeg("-0",2),10))!=OK) {
       WY_THROW(r);
     }
     if(tnn!=NumType(0)) {
       WY_THROW(r);
     }

     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(ns1,sizeof(ns1)-1),10))!=OK) {
       WY_THROW(r);
     }
     if(endptr-ns1!=sizeof(ns1)-1) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-52)) {
       WY_THROW( WyRet() );
     }

     const char ns2[]="-0x041";
     radix=0;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(ns2,sizeof(ns2)-1),radix))!=OK) {
       WY_THROW(r);
     }
     if(endptr-ns2!=sizeof(ns2)-1) {
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-0x41)) {
       WY_THROW( WyRet() );
     }
  
     const char ns3[]="-041";
     radix=0;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(ns3,sizeof(ns3)-1),radix))!=OK) {
       WY_THROW(r);
     }
     if(endptr-ns3!=sizeof(ns3)-1) {
       WY_THROW( WyRet() );
     }
     if(radix!=8) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-041)) {
       WY_THROW( WyRet() );
     }
  
     const char ns4[]="-41";
     radix=5;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(ns4,sizeof(ns4)-1),radix))!=OK) {
       WY_THROW(r);
     }
     if(endptr-ns4!=sizeof(ns4)-1) {
       WY_THROW( WyRet() );
     }
     if(radix!=5) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-21)) {
       WY_THROW( WyRet() );
     }
   }
 }

 // chk error Wy::_strnum(...)
 {
   NumType tnn=3;
   const char* buf=0;
   const char* endptr=(const char*)1;
   int radix;
   try {
     r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,1),3);
     WY_THROW(r);
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };

   buf="73!";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,2),1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if(endptr-buf!=0) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,2),37))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if(endptr-buf!=0) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),10))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=2) {
     WY_THROW( WyRet() );
   }
   if(tnn!=73) {
     WY_THROW( WyRet() );
   }

   buf="+";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,1),2))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if(endptr-buf!=0) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xx";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),0))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xg";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),0))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xx";
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=16;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   buf="0xg";
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   radix=0;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=0;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=16;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   buf="0xfg";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=3) {
     WY_THROW( WyRet() );
   }
   if(tnn!=0xf) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   if((r=Wy::_strnum(tnn,&endptr,WyCSeg("fffffffffffffffffffffff"),16))!=Wym_ERANGE) {
     WY_THROW(r);
   }
   if(endptr-buf<(int)(2*sizeof(NumType))) {
     WY_THROW( WyRet() );
   }
   if(tnn<15) {
     WY_THROW( WyRet() );
   }

   if(std::numeric_limits<NumType>::is_signed) {
     buf=(const char*)0;
     tnn=4;
     try {
       r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,1),3);
       WY_THROW(r);
     }
     catch(const WyCSeg::Reply& e) {
       if(e!=Wym_EFAULT) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
  
     buf="-7";
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,2),37))!=Wym_EINVAL) {
       WY_THROW(r);
     }
     if(endptr-buf!=0) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }

     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,3),8))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-7)) {
       WY_THROW( WyRet() );
     }

     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,2),1))!=Wym_EINVAL) {
       WY_THROW(r);
     }
     if(endptr-buf!=0) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }

     buf="-0xx";
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),0))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xg";
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),0))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xx";
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=16;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xg";
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     radix=0;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=0;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=16;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=2) {
       WY_THROW( WyRet() );
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xfg";
     tnn=99;
     if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,5),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(endptr-buf!=4) {
       WY_THROW( WyRet() );
     }
     if(tnn+0xf!=0) {          // compiler complains "tnn!=-0xf"
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }

     if((r=Wy::_strnum(tnn,&endptr,WyCSeg("-fffffffffffffffffffffff"),16))!=Wym_ERANGE) {
       WY_THROW(r);
     }
     if(endptr-buf<(int)(2*sizeof(NumType))) {
       WY_THROW( WyRet() );
     }
     if(tnn>0) {
       WY_THROW( WyRet() );
     }
   }
 }
};

template <typename NumT>
static void ck_sqrt(void)
{
 if(std::numeric_limits<NumT>::is_signed) {
   return;
 }
 if(Wy::_sqrt(NumT(0))!=0) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(1))!=1) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(99))!=9) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(100))!=10) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(120))!=10) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(121))!=11) {
   WY_THROW( WyRet() );
 }
 for(int i=0; i<=255; ++i) {  // chk i in char range
   NumT res=Wy::_sqrt((NumT)i);
   int q(res),t2(q*q);
   if(t2>i) {
     WY_THROW( WyRet() );
   }
   if(t2+2*q+1<=i) {
     WY_THROW( WyRet() );
   }
 }

 for(NumT i=std::numeric_limits<NumT>::max(); 
          i>=std::numeric_limits<NumT>::max()-254; --i) {
   NumT res=Wy::_sqrt(i);
   NumT q(res),t2(q*q);
   if(t2>i) {
     WY_THROW( WyRet() );
   }
   if(t2+2*q<=i-1) {
     std::cout << t2 << ", " << q << ", " << i << std::endl;
     std::cout << t2+2*q+1 << std::endl;
     WY_THROW( WyRet() );
   }
 }
};

template <typename T>
static void ck_num_frm_str(void)
{
 struct ChkSmp {
   T num;
   const char* nstr;
   size_t nstr_size;
   int    radix;
   void verify(void) const 
     {
      try {
        WyRet r;
        const char* endptr;
        T a1;
        if((r=Wy::_strnum(a1,&endptr,WyCSeg(nstr,nstr_size),radix))!=OK) {
          WY_THROW(r);
        }
        if(a1!=num) {
          WY_THROW( WyRet() );
        }
        if(endptr-nstr!=int(nstr_size)) {
          WY_THROW( WyRet() );
        }
      }
      catch(...) {
        if((std::numeric_limits<T>::max()==std::numeric_limits<char>::max())||
           (std::numeric_limits<T>::max()==std::numeric_limits<unsigned char>::max())) {
          std::cerr << "ChkSmp{ T(" << (int)num << "),\""
                    << nstr << "\"," << nstr_size << ","
                    << radix << "}" << std::endl;
        } else {
          std::cerr << "ChkSmp{ T(" << num << "),\""
                    << nstr << "\"," << nstr_size << ","
                    << radix << "}" << std::endl;
        }
        throw;
      };
     }
 } const chk_tab[]={
  {T(0),"0",1,10}, {T(0),"+0",2,10}, {T(0),"-0",2,10},
  {T(0),"0.0",3,10},{T(0),"+0.0",4,10},{T(0),"-0.0",4,10},
  {T(0),"00.4",3,10},{T(0),"+00.0",5,10},{T(0),"-00.0",5,10},
  {T(0),"0.00",4,10},{T(0),"+0.00",5,10},{T(0),"-0.00",5,10},
  {T(0),"0.00e0",6,10},{T(0),"+0.00e0",7,10},{T(0),"-0.00e0",7,10},

  {T(1.0),"1",1,10},{T(1.0),"+1",2,10},{T(-1.0),"-1",2,10},
  {T(1.0),"1.0",3,10},{T(1.0),"+1.0",4,10},{T(-1.0),"-1.0",4,10},
  {T(1.0),"01.0",4,10},{T(1.0),"+01.0",5,10},{T(-1.0),"-01.0",5,10},
  {T(1.0),"1.00",4,10},{T(1.0),"+1.00",5,10},{T(-1.0),"-1.00",5,10},
  {T(1.0),"1.00e0",6,10},{T(1.0),"+1.00e0",7,10},{T(-1.0),"-1.00e0",7,10},
  {T(1.0),"1.00e+0",7,10},{T(1.0),"+1.00e+0",8,10},{T(-1.0),"-1.00e+0",8,10},
  {T(1.0),"1.00e-0",7,10},{T(1.0),"+1.00e-0",8,10},{T(-1.0),"-1.00e-0",8,10},

  {T(10.0),"1.00e1",6,10},{T(10.0),"+1.00e1",7,10},{T(-10.0),"-1.00e1",7,10},
  {T(10.0),"1.00e+1",7,10},{T(10.0),"+1.00e+1",8,10},{T(-10.0),"-1.00e+1",8,10},
  {T(0.1),"1.00e-1",7,10},{T(0.1),"+1.00e-1",8,10},{T(-0.1),"-1.00e-1",8,10},
 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
   chk_tab[i].verify();
 }

 // chk error
 {
   T tnn;
   WyRet r;
   const char *buf,*endptr;

   buf="12.34ee";
   tnn=99;
   if((r=Wy::_strnum(tnn,&endptr,WyCSeg(buf,7)))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(endptr-buf!=5) {
     WY_THROW( WyRet() );
   }
   if(Wy::wy_roughly_equ(tnn,12.34)==false) {
     WY_THROW( WyRet() );
   }
 }
};

template <typename NumType>
static void ck_num_float(void)
{
 ck_num_frm_str<NumType>();
};

template <typename NumType>
static void ck_num_integer(void)
{
 ck_uty<NumType>();
};

static void ck_examp1(void)
{
 // Evaluate the code to parse number " 12/23/44.4  56,456.21 4k22"
 //
 const char nstr[]=" 12/23/44.4  56,456.21 4k22";
 const char* const nstr_end=nstr+sizeof(nstr)-1;
 WyRet r;
 int v_int;
 double v_double;
 unsigned int v_uint;
 const char* endptr;
 WyCSeg cs(nstr+1,sizeof(nstr)-3);

 if((r=Wy::_strnum(v_int,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_int!=12) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+3) {
   WY_THROW( WyRet() );
 }
 
 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_uint,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_uint!=23) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+6) {
   WY_THROW( WyRet() );
 }

 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_double,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if((v_double<44.39)||(v_double>44.41)) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+11) {
   WY_THROW( WyRet() );
 }

 ++endptr;
 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_int,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_int!=56) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+15) {
   WY_THROW( WyRet() );
 }

 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_double,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if((v_double<456.209)||(v_double>456.211)) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+22) {
   WY_THROW( WyRet() );
 }
 v_double+= 1000.0*v_int;   // i.e. 56456.21

 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_uint,&endptr,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_uint!=4) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+24) {
   WY_THROW( WyRet() );
 }

 ++endptr;
 cs.reset(endptr,nstr_end-endptr);
 if((r=Wy::_strnum(v_uint,&endptr,cs))!=OK) {
   WY_THROW(r);
 }
 if(v_uint!=22) {
   WY_THROW( WyRet() );
 }
 if(endptr!=nstr+27) {
   WY_THROW( WyRet() );
 }
};

void ck_num(void)
{
 ck_strnum();

 ck_num_integer<char>(); 
 ck_num_integer<short>();
 ck_num_integer<int>();
 ck_num_integer<long>();
 ck_num_integer<long long>();

 ck_num_integer<signed char>();
 ck_num_integer<signed short>();
 ck_num_integer<signed int>();
 ck_num_integer<signed long>();
 ck_num_integer<signed long long>();

 ck_num_integer<unsigned char>();
 ck_num_integer<unsigned short>();
 ck_num_integer<unsigned int>();
 ck_num_integer<unsigned long>();
 ck_num_integer<unsigned long long>();

 ck_num_float<float>();
 ck_num_float<double>();

 ck_sqrt<unsigned char>();
 ck_sqrt<unsigned short>();
 ck_sqrt<unsigned int>();
 ck_sqrt<unsigned long>();
 ck_sqrt<unsigned long long>();

 ck_examp1();
};
