/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004
*/

#include "wystr.h"
#include <iostream>
#include <cstdlib>
#include <cmath>

#if WYSTR_VERSION!=30
#error Test code is for WYSTR_VERSION 30
#endif

#ifdef WY_DEBUG
#include <string>
#define StdNPos std::string::npos
#endif

static bool chk_default(const WyStr& str)
{
 if(str._capacity()<WyStr::min_capacity()) {
   WY_THROW( WyRet() );
 }
 if(str.c_str()!=str.data()) {
   WY_THROW( WyRet() );
 }
 static const WyStr S0;
 if(str.is_default()==false) {
   return(false);
 }
 if(str==S0) {
 } else {
   return(false);
 }
 if(str!=S0) {
   return(false);
 }
 if(str.size()!=0) {
   return(false);
 }
 if(str.begin()!=&str[0]) {
   return(false);
 }
 if(str.begin()!=str.end()) {
   return(false);
 }

 const char* cptr;
 cptr=str.data();
 if(cptr==0) {
   WY_THROW(WyRet() ); 
 }
 if(*cptr!=0) {
   return(false);
 }
 cptr=str.c_str();
 if(cptr==0) {
   WY_THROW(WyRet() ); 
 }
 if(*cptr!=0) {
   return(false);
 }
 return(true);
};

static bool chk_not_default(const WyStr& str)
{
 if(str._capacity()<WyStr::min_capacity()) {
   WY_THROW( WyRet() );
 }
 if(str.c_str()!=str.data()) {
   WY_THROW( WyRet() );
 }
 static const WyStr S0;
 if(str.is_default()) {
   return(false);
 }
 if(str!=S0) {
 } else {
   return(false);
 }
 if(str==S0) {
   return(false);
 }
 if(str.size()<=0) {
   return(false);
 }
 if(str.begin()==str.end()) {
   return(false);
 }

 const char* cptr;
 cptr=str.data();
 if(cptr==0) {
   WY_THROW(WyRet() ); 
 }
 if(cptr[str.size()]!=0) {
   WY_THROW(WyRet() ); 
 }

 cptr=str.c_str();
 if(cptr==0) {
   WY_THROW(WyRet() ); 
 }
 if(cptr[str.size()]!=0) {
   WY_THROW(WyRet() ); 
 }
 return(true);
};

#define CHK_DEFAULT(expr) if(chk_default(expr)==false) { WY_THROW(WyRet() ); };
#define CHK_NOT_DEFAULT(expr) if(chk_not_default(expr)==false) { WY_THROW(WyRet() ); };

static void t1(void)
{
 WyRet r;

 // default constructor
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);

  WyStr tmp2(tmp);
  CHK_DEFAULT(tmp2);

  WyStr tmp3;
  CHK_DEFAULT(tmp3);
  WyStr tmp4(tmp3);
  CHK_DEFAULT(tmp4);
 }

 // copy/from cstr constructor
 {
  WyStr tmp("abc");
  CHK_NOT_DEFAULT(tmp);
  if(tmp.size()!=3) {
    WY_THROW(WyRet() );
  }
  if((tmp[0]!='a')||(tmp[1]!='b')||(tmp[2]!='c')||(tmp[3]!=0)) {
    WY_THROW(WyRet() );
  }
  
  WyStr tmp2(tmp);

  if(tmp==tmp2) {
  } else {
    WY_THROW(WyRet() );
  }
  if(tmp!=tmp2) {
    WY_THROW(WyRet() );
  } else {
  }
  if(std::memcmp(tmp.data(),tmp.c_str(),tmp.size()+1)!=0) {
    WY_THROW(WyRet() );
  }
  if(std::memcmp(tmp.data(),tmp2.data(),tmp.size()+1)!=0) {
    WY_THROW(WyRet() );
  }

  WyStr tmp3("");
  CHK_DEFAULT(tmp3);

  WyStr tmp4(tmp2);

  if(tmp4==tmp2) {
  } else {
    WY_THROW(WyRet() );
  }
  if(tmp4!=tmp2) {
    WY_THROW(WyRet() );
  } else {
  }
 }

 // WyStr(const WyStr&,size_t)
 {
  WyStr Frm("12345");
  WyStr tmp(Frm.cseg(0,100));
  if(tmp!=Frm) {
    WY_THROW(WyRet() );
  }

  WyStr tmp1(Frm.cseg(1,100));
  if(tmp1!="2345") {
    WY_THROW(WyRet() );
  }

  WyStr tmp2(Frm.cseg(Frm.size(),1));
  if(tmp2.is_default()==false) {
    WY_THROW(WyRet() );
  }
 }

 // WyStr(const WyStr&,size_t,size_t)
 {
  WyStr S_abcd("abcd");
  WyStr tmp(S_abcd.cseg(0,0));
  CHK_DEFAULT(tmp);

  WyStr tmp1(S_abcd.cseg(1,2));
  if(tmp1!="bc") {
    WY_THROW(WyRet() );
  }

  WyStr tmp2(S_abcd.cseg(2,5));
  if(tmp2!="cd") {
    WY_THROW(WyRet() );
  }
 }

 // WyStr(const char*,size_t)
 {
  WyStr tmp( WyCSeg("abcd",size_t(0)) );
  CHK_DEFAULT(tmp);

  WyStr tmp1( WyCSeg("abcd",1) );
  if(tmp1!="a") {
    WY_THROW(WyRet() );
  }

  WyStr tmp2( WyCSeg("abcd",2) );
  if(tmp2!="ab") {
    WY_THROW(WyRet() );
  }
 }

 // WyStr(size_t,char)
 {
  WyStr tmp(size_t(0),char('a'));
  CHK_DEFAULT(tmp);

  WyStr tmp1(size_t(1),char('b'));
  if(tmp1!="b") {
    WY_THROW(WyRet() );
  }

  WyStr tmp2(size_t(100),char('c'));
  CHK_NOT_DEFAULT(tmp2);
  for(size_t i=0; i<tmp2.size(); ++i) {
    if(tmp2[i]!='c') {
      WY_THROW(WyRet() );
    }
  }
  if(tmp2[tmp2.size()]!=0) {
    WY_THROW(WyRet() );
  }
 }

 // operator ==(const char*)
 {
  const char* CPtr="4567";
  WyStr tmp(CPtr);
  if(tmp==(const char*)CPtr) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tmp!=(const char*)CPtr) {
    WY_THROW( WyRet() );
  }
  if(tmp==(const char*)"uuuuu") {
    WY_THROW( WyRet() );
  }
  if(tmp!=(const char*)"uuuuu") {
  } else {
    WY_THROW( WyRet() );
  }
 }

 // operator[]
 {
  WyStr tmp0;
  CHK_DEFAULT(tmp0);
  if(tmp0[0]!=0) {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp0);

  WyStr tmp1("123");
  CHK_NOT_DEFAULT(tmp1);
  char* tp=&tmp1[0];
  if((tp[0]!='1')||(tp[1]!='2')||(tp[2]!='3')||(tp[3]!=0)) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp1);

  if((tmp1[0]!='1')||(tmp1[1]!='2')||(tmp1[2]!='3')||(tmp1[3]!=0)) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp1);

  if(std::memcmp(tmp0.data(),tmp0.c_str(),tmp0.size()+1)!=0) {
    WY_THROW(WyRet() );
  }
 }

 // at
 {
  WyStr tmp0;
  CHK_DEFAULT(tmp0);

  WyStr tmp1("123");
  CHK_NOT_DEFAULT(tmp1);
  char* tp=&tmp1.at(0);
  if((tp[0]!='1')||(tp[1]!='2')||(tp[2]!='3')||(tp[3]!=0)) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp1);

  if((tmp1.at(0)!='1')||(tmp1.at(1)!='2')||(tmp1.at(2)!='3')) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp1);
 }

 // begin/end
 {
  WyStr tmp0;
  CHK_DEFAULT(tmp0);
  if(tmp0.begin()!=&tmp0[0]) {
    WY_THROW(WyRet());
  }
  if(tmp0.begin()!=tmp0.end()) {
    WY_THROW(WyRet());
  }

  WyStr tmp1("abcx");
  CHK_NOT_DEFAULT(tmp1);
  if(tmp1.end()==&tmp1[0]) {
    WY_THROW(WyRet());
  }
  if(tmp1.begin()==tmp1.end()) {
    WY_THROW(WyRet());
  }
  if((tmp1.end()-tmp1.begin())!=(int)tmp1.size()) {
    WY_THROW(WyRet());
  }
 }

 // reset(const WyStr&)
 {
  WyStr tmp,tmp0;
  WyStr tmp1("rst");

  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);

  if((r=tmp.reset(tmp))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(tmp0))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(tmp1))!=OK) {
    WY_THROW(r);
  }
  CHK_NOT_DEFAULT(tmp);
  if(tmp!=WyStr("rst")) {
    WY_THROW(WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset(tmp))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
 }

 // reset(const WyStr&,size_t)
 {
  WyStr Frm("12345"),tmp;
  
  if((r=tmp.reset(Frm.cseg(0)))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!=Frm) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset(Frm.cseg(1,100)))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!="2345") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset(Frm.cseg(Frm.size(),1)))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW(WyRet() );
  }

  try {
    r=tmp.reset( Frm.cseg(1000,1) );
    WY_THROW( WyRet() );
  }
  catch(const WyStr::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW(r);
    }
    // FALL_THROUGH
  };
 }

 // reset(const WyStr&,size_t,size_t)
 {
  WyStr tmp,tmp0;
  WyStr tmp1("rst");

  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);

  if((r=tmp.reset(tmp0.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(tmp0.cseg(0,2)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(tmp1.cseg(0,5)))!=OK) {
    WY_THROW(r);
  }
  CHK_NOT_DEFAULT(tmp);
  if(tmp!=WyStr("rst")) {
    WY_THROW(WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset(tmp))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }

  {
   const char* TStr="abcdefg1234567";
   const WyStr TmpA(TStr);
   if((r=tmp.reset(TmpA))!=OK) {
     WY_THROW(r);
   }
   if(tmp!=TStr) {
     WY_THROW(WyRet() );
   }
   
   if((r=tmp.reset(tmp.cseg(1,100)))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=tmp.reset(tmp.cseg(tmp.size()-1,1)))!=OK) {
     // This is imp. feature, src/des on the same buffer though
     WY_THROW(r);
   }
  }
 }

 // reset(const char*,size_t)
 {
  WyStr tmp;
  const char* cstr("rst");

  CHK_DEFAULT(tmp);

  if((r=tmp.reset( WyCSeg("a",size_t(0))) )!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset( WyCSeg("a",1)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(r);
  }

  if((r=tmp.reset( WyCSeg(cstr,2)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="rs") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset( WyCSeg(cstr,3)) )!=OK) {
    WY_THROW(r);
  }
  CHK_NOT_DEFAULT(tmp);
  if(tmp!="rst") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset( WyStr(tmp.cseg(1,1)) ) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="s") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset("abc123"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc123") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.reset( WyStr(tmp.cseg(1,5))) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="bc123") {
    WY_THROW(WyRet() );
  }
 }

 // reset(const char*)
 {
  WyStr tmp;
  const char* cstr("rst");
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(""))!=OK) {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(cstr))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!="rst") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset( WyStr(tmp.cseg(0)).c_str()))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!="rst") {
    WY_THROW(WyRet() );
  }

  const char* c2="123455344343g";
  if((r=tmp.reset(c2))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!=c2) {
    WY_THROW(WyRet() );
  }
 }

 // reset(size_t,char)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(0,'a'))!=OK) {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.reset(1,'a'))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }

  size_t ca=tmp._capacity()+1;
  if((r=tmp.reset(ca,'b'))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp.size()!=ca) {
    WY_THROW(WyRet() );
  }
  for(size_t i=0; i<ca; ++i) {
    if(tmp[i]!='b') {
      WY_THROW(WyRet() );
    }
  }
  // reallocate
  {
   const char* PrePtr=tmp.data();
   if((r=tmp.reset(WyStr::min_capacity(),'a'))!=OK) {
     WY_THROW( WyRet() );
   }
   if(tmp.data()!=PrePtr) {
     WY_THROW( WyRet() );
   }
   if((r=tmp.reset(tmp._capacity()+1,'x'))!=OK) {
     WY_THROW( WyRet() );
   }
   if(tmp.data()==PrePtr) {
     WY_THROW( WyRet() );
   }
  }
 }

 // operator =(const char*)
 {
  WyStr tmp;
  const char* cstr("rst");
  CHK_DEFAULT(tmp);

  tmp="";
  CHK_DEFAULT(tmp);

  tmp=cstr;
  if(tmp!="rst") {
    WY_THROW(WyRet() );
  }

  tmp= WyStr(tmp.cseg(0)).c_str();
  if(tmp!="rst") {
    WY_THROW(WyRet() );
  }

  const char* c2="123455344343g";
  tmp=c2;
  if(tmp!=c2) {
    WY_THROW(WyRet() );
  }
 }

 // operator =(const WyStr&)
 {
  WyStr tmp,tmp0;
  WyStr tmp1("rst");

  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);

  tmp=tmp;
  CHK_DEFAULT(tmp);

  tmp=tmp0;
  CHK_DEFAULT(tmp);

  tmp=tmp1;
  CHK_NOT_DEFAULT(tmp);
  if(tmp!=WyStr("rst")) {
    WY_THROW(WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
 }

 // swap
 {
  WyStr tmp,tmp0;
  WyStr tmp1("ab"),tmp2("1234");
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp2);

  tmp.swap(tmp0);
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);

  tmp.swap(tmp0);
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);

  tmp.swap(tmp1);
  if(tmp!="ab") {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp1);
  tmp.swap(tmp1);
  if(tmp1!="ab") {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp0);
  
  tmp1.swap(tmp2);
  if(tmp1!="1234") {
    WY_THROW(WyRet() );
  }
  if(tmp2!="ab") {
    WY_THROW(WyRet() );
  }
  tmp1.swap(tmp2);
  if(tmp1!="ab") {
    WY_THROW(WyRet() );
  }
  if(tmp2!="1234") {
    WY_THROW(WyRet() );
  }
 }

 // append(const WyStr&)
 {
  WyStr tmp,tmp0,tmp00("");
  const WyStr tmp1("ab"),tmp2("1234");
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp2);

  if((r=tmp.append(tmp))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append(tmp00))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append(tmp1))!=OK) {
    WY_THROW(r);
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp);

  if((r=tmp.append(tmp1))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp2))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abab1234") {
    WY_THROW(WyRet() );
  }

  WyStr tmp5("abc");
  if((r=tmp5.append(tmp5))!=OK) {
    WY_THROW(r);
  }
  if(tmp5!="abcabc") {
    WY_THROW(WyRet() );
  }
 }

 // operator +=(const WyStr&)
 {
  WyStr tmp,tmp0,tmp00("");
  const WyStr tmp1("ab"),tmp2("1234");
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp2);

  tmp+=tmp;
  CHK_DEFAULT(tmp);
  
  tmp+=tmp00;
  CHK_DEFAULT(tmp);

  tmp+=tmp1;
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp);

  tmp+=tmp1;
  if(tmp!="abab") {
    WY_THROW(WyRet() );
  }
  tmp+=tmp2;
  if(tmp!="abab1234") {
    WY_THROW(WyRet() );
  }

  WyStr tmp5("abc");
  tmp5+=tmp5;
  if(tmp5!="abcabc") {
    WY_THROW(WyRet() );
  }

 }

 // append(const WyStr&,size_t)
 {
  WyStr tmp,tmp0,tmp00("");
  const WyStr tmp1("ab"),tmp2("1234");
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp2);

  if((r=tmp.append(tmp.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append(tmp00.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append(tmp1.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp);

  if((r=tmp.append(tmp1.cseg(1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abb") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp2.cseg(2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abb34") {
    WY_THROW(WyRet() );
  }

  WyStr tmp5("abc");
  if((r=tmp5.append(tmp5.cseg(3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp5!="abc") {
    WY_THROW(WyRet() );
  }
 }

 // append(const WyStr&,size_t,size_t)
 {
  WyStr tmp,tmp0,tmp00("");
  const WyStr tmp1("ab"),tmp2("1234");
  CHK_DEFAULT(tmp);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp2);

  if((r=tmp.append(tmp.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append(tmp00.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append(tmp1.cseg(0,2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!=tmp1) {
    WY_THROW(WyRet() );
  }
  CHK_NOT_DEFAULT(tmp);

  if((r=tmp.append(tmp1.cseg(0,3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp2.cseg(2,3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abab34") {
    WY_THROW(WyRet() );
  }

  WyStr tmp5("abc");
  if((r=tmp5.append(tmp5.cseg(0,4)))!=OK) {
    WY_THROW(r);
  }
  if(tmp5!="abcabc") {
    WY_THROW(WyRet() );
  }
 }

 // append(const char*,size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append( WyCSeg("",size_t(0))) )!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append( WyCSeg("a",1)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append( WyCSeg(tmp.data(),1)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.append( WyCSeg("b",1) ))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append( WyCSeg("",size_t(0)) ))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append( WyCSeg("1234",4)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab1234") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append( WyCSeg(tmp.data(),5)) )!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab1234aab12") {
    WY_THROW(WyRet() );
  }
 }

 // append(const char* cstr)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append( WyCSeg("") ))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append("a"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp.data()))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.append("b"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(""))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append("1234"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab1234") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp.data()))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab1234aab1234") {
    WY_THROW(WyRet() );
  }
 }

 // operator +=(const char* cstr)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  tmp+="";
  CHK_DEFAULT(tmp);

  tmp+="a";
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  tmp+=tmp.data();
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  tmp+="b";
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  tmp+="";
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  tmp+="1234";
  if(tmp!="aab1234") {
    WY_THROW(WyRet() );
  }
  tmp+=tmp.data();
  if(tmp!="aab1234aab1234") {
    WY_THROW(WyRet() );
  }
 }

 // append(size_t,char)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append(0,'a'))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.append(1,'a'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(1,tmp[0]))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.append(1,'b'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(0,'x'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(4,'4'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab4444") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(5,tmp[2]))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab4444bbbbb") {
    WY_THROW(WyRet() );
  }
 }

 // append(char)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.append('a'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp[0]))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.append('b'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append('4'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab4") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.append(tmp[2]))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aab4b") {
    WY_THROW(WyRet() );
  }
 }

 // operator +=(char)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  tmp+='a';
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  tmp+=tmp[0];
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }

  tmp+='b';
  if(tmp!="aab") {
    WY_THROW(WyRet() );
  }
  tmp+='4';
  if(tmp!="aab4") {
    WY_THROW(WyRet() );
  }
  tmp+=tmp[2];
  if(tmp!="aab4b") {
    WY_THROW(WyRet() );
  }
 }

 // insert(size_t,const WyStr&)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.insert(0,WyStr("")))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0,WyStr("a")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,WyStr("z")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.insert(2,WyStr("b")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,WyStr("ff")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffzab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(3,WyStr("12345678")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz12345678ab") {
    WY_THROW(WyRet() );
  }
 }

 // insert(size_t,const WyStr&,size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  const WyStr S_a("a");
  if((r=tmp.insert(0,S_a.cseg(1)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0,S_a.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  const WyStr S_z("z");
  if((r=tmp.insert(0,S_z.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za") {
    WY_THROW(WyRet() );
  }

  const WyStr S_b("b");
  if((r=tmp.insert(2,S_b.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zab") {
    WY_THROW(WyRet() );
  }
  const WyStr S_1234("1234");
  if((r=tmp.insert(2,S_1234.cseg(2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za34b") {
    WY_THROW(WyRet() );
  }
 }

 // insert(size_t,const WyStr&,size_t,size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
 
  WyStr S_a("a"); 
  if((r=tmp.insert(0,S_a.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0,S_a.cseg(0,1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  WyStr S_z("z"); 
  if((r=tmp.insert(0,S_z.cseg(0,1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za") {
    WY_THROW(WyRet() );
  }

  WyStr S_b("b"); 
  if((r=tmp.insert(2,S_b.cseg(0,1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zab") {
    WY_THROW(WyRet() );
  }
  WyStr S_ff("ff"); 
  if((r=tmp.insert(0,S_ff.cseg(0,2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffzab") {
    WY_THROW(WyRet() );
  }
  WyStr S_1234("1234"); 
  if((r=tmp.insert(3,S_1234.cseg(0,4)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz1234ab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(8,S_1234.cseg(1,4)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz1234a234b") {
    WY_THROW(WyRet() );
  }
 }

 // insert(size_t,const char*)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.insert(0,""))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0,"a"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,"z"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.insert(2,"b"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,"ff"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffzab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(3,"12345678"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz12345678ab") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.reset("abcd12345"))!=OK) {
    WY_THROW(r);
  }
  /* self-operate
  if((r=tmp.insert(3, WyCSeg(tmp.data()+4,3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc123d12345") {
    std::cout << tmp.data() << std::endl;
    WY_THROW(WyRet() );
  }*/
 }

 // insert(size_t,const char*,size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.insert(0, WyCSeg("a",size_t(0))))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0, WyCSeg("a",1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="a") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0, WyCSeg("z",1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="za") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.insert(2, WyCSeg("b",1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0, WyCSeg("ff",2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffzab") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(3, WyCSeg("1234",4)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz1234ab") {
    WY_THROW(WyRet() );
  }
 }

 // insert(size_t,size_t,char)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.insert(0,0,'e'))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  if((r=tmp.insert(0,1,'e'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="e") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,1,'z'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ze") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.insert(2,1,'b'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="zeb") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(0,2,'f'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffzeb") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.insert(3,4,'1'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ffz1111eb") {
    WY_THROW(WyRet() );
  }
 }

 // erase(size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.erase(0))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  WyStr tmp0("1234567");
  WyStr tmp1(tmp0);
  if((r=tmp1.erase(tmp1.size()))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!=tmp0) {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(tmp1.size()-1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!=WyStr( WyCSeg(tmp0.data(),tmp0.size()-1) )) {
    WY_THROW(WyRet() );
  }

  if((r=tmp1.erase(2))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="12") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="1") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(0))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="") {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp1);
 }

 // erase(size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.erase(0))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  WyStr tmp0("1234567");
  WyStr tmp1(tmp0);
  if((r=tmp1.erase(tmp1.size()))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!=tmp0) {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(tmp1.size()-1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!=WyStr( WyCSeg(tmp0.data(),tmp0.size()-1)) ) {
    WY_THROW(WyRet() );
  }

  if((r=tmp1.erase(2))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="12") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="1") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(0))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="") {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp1);
 }

 // erase(size_t,size_t)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  
  if((r=tmp.erase(0,0))!=OK) {
    WY_THROW(r);
  }
  if((r=tmp.erase(0,1))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);

  WyStr tmp0("1234567");
  WyStr tmp1(tmp0);
  if((r=tmp1.erase(tmp1.size(),1))!=OK) {
    WY_THROW(r);
  }
  if(tmp0!=tmp1) {
    WY_THROW(WyRet() );
  }

  if((r=tmp1.erase(3,1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="123567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(5,1))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="12356") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(0,3))!=OK) {
    WY_THROW(r);
  }
  if(tmp1!="56") {
    WY_THROW(WyRet() );
  }
  if((r=tmp1.erase(0,10))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
 }

 // resize
 {
  WyStr tmp;
  if((r=tmp.resize(0))!=OK) {
    WY_THROW(WyRet() );
  }
  CHK_DEFAULT(tmp);
  if((r=tmp.resize(7))!=OK) {
    WY_THROW(WyRet() );
  }

  {
   const char ca[8]={char(),char(),char(),char(),char(),char(),char(),0};
   if(tmp.size()!=7) {
     WY_THROW(WyRet() );
   }
   if((std::memcmp(tmp.data(),ca,sizeof(ca)))!=0) {
     WY_THROW(WyRet() );
   }
  }

  if((r=tmp.resize(2,'c'))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp.size()!=2) {
    WY_THROW(WyRet() );
  }
  if((tmp[0]!=char())||(tmp[1]!=char())||(tmp[2]!=0)) {
    WY_THROW(WyRet() );
  }
  if((r=tmp.resize(3,'c'))!=OK) {
    WY_THROW(WyRet() );
  }
  if(tmp.size()!=3) {
    WY_THROW(WyRet() );
  }
  if(tmp[2]!='c') {
    WY_THROW(WyRet() );
  }

  r=tmp.resize(tmp._capacity()+2000);
  if(r!=OK) {
    WY_THROW(r);
  }
  while(tmp.size()) {
    r=tmp.resize(tmp.size()-1);
    if(r!=OK) {
      // shrink by resize never cause error
      WY_THROW(r);
    }
  }
  CHK_DEFAULT(tmp);
  if(tmp!="") {
    WY_THROW( WyRet() );
  }
 }

 // replace(size_t,size_t,const WyStr&)
 {
  WyStr tmp;
  if((r=tmp.replace(0,4,WyStr()))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  if((r=tmp.replace(0,4,WyStr("abc")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1,WyStr("")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(2,2,WyStr("")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(1,0,WyStr("x")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axc") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3,WyStr("")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="") {
    WY_THROW(WyRet() );
  }

  // AUX: operator + (char,const WyStr&)
  {
    const WyStr Tmp("gh");
    if((Tmp+'f')!="ghf") {
      WY_THROW(WyRet() );
    }
    if(('f'+Tmp)!="fgh") {
      WY_THROW(WyRet() );
    }
  }

/* self-operate
  {
   WyStr a("abcde");
   if((r=a.replace(2,1,a))!=OK) {
     WY_THROW(r);
   }
   if(a!="ababcdede") {
    WY_THROW(WyRet() );
  }
  }
*/
  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,2,WyStr("ab")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3,WyStr("xy")))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab45xy") {
    WY_THROW(WyRet() );
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,1, WyCSeg(rstr.data(),rstr.size())))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+6) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   size_t t2=tmp.size()-3;
   for(size_t i=3; i<t2; ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if((tmp[t2+0]!='5')||(tmp[t2+1]!='6')||(tmp[t2+2]!='7')) {
     WY_THROW(WyRet() );
   }
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,100,rstr))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+3) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   for(size_t i=3; i<tmp.size(); ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if(tmp[tmp.size()]!=0) {
     WY_THROW(WyRet() );
   }
  }
 }

 // replace(size_t,size_t,const WyStr&,size_t)
 {
  WyStr tmp;
  const WyStr S_empty;
  if((r=tmp.replace(0,4,S_empty.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  const WyStr S_labc("labc");
  if((r=tmp.replace(0,4,S_labc.cseg(1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1,S_empty.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(2,2,S_empty.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  WyStr S_x("x");
  if((r=tmp.replace(1,0,S_x.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axc") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3,S_empty.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="") {
    WY_THROW(WyRet() );
  }

/* self-operate
  {
   WyStr a("abcde");
   if((r=a.replace(2,1,a,0))!=OK) {
     WY_THROW(r);
   }
   if(a!="ababcdede") {
    WY_THROW(WyRet() );
  }
  }
*/

  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  WyStr S_xab("xab");
  WyStr S_xy("xy");
  if((r=tmp.replace(1,2,S_xab.cseg(1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3,S_xy.cseg(0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab45xy") {
    WY_THROW(WyRet() );
  }
 }

 // replace(size_t,size_t,const WyStr&,size_t,size_t)
 {
  WyStr tmp;
  const WyStr S_empty, S_labcd("labcd"), S_xx("xx"), S_x("x");

  if((r=tmp.replace(0,4,S_empty.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  if((r=tmp.replace(0,4,S_labcd.cseg(1,3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1,S_xx.cseg(1,0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(2,2,S_empty.cseg(0,0)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(1,0,S_x.cseg(0,2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axc") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3,S_empty.cseg(0,1)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="") {
    WY_THROW(WyRet() );
  }

/* self-operate
  {
   WyStr a("abcde");
   if((r=a.replace(2,1,a,0,a.size()))!=OK) {
     WY_THROW(r);
   }
   if(a!="ababcdede") {
    WY_THROW(WyRet() );
  }
  }
*/

  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  WyStr S_xabx("xabx"),S_xy("xy");
  if((r=tmp.replace(1,2,S_xabx.cseg(1,2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3,S_xy.cseg(0,2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab45xy") {
    WY_THROW(WyRet() );
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,1,rstr.cseg(0,rstr.size())))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+6) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   size_t t2=tmp.size()-3;
   for(size_t i=3; i<t2; ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if((tmp[t2+0]!='5')||(tmp[t2+1]!='6')||(tmp[t2+2]!='7')) {
     WY_THROW(WyRet() );
   }
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,100,rstr.cseg(0,rstr.size())))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+3) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   for(size_t i=3; i<tmp.size(); ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if(tmp[tmp.size()]!=0) {
     WY_THROW(WyRet() );
   }
  }
 }

 // replace(size_t,size_t,const char*,size_t)
 {
  WyStr tmp;
  if((r=tmp.replace(0,4, WyCSeg("a",size_t(0))))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  if((r=tmp.replace(0,4, WyCSeg("abc",3)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1, WyCSeg("",size_t(0))))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(2,2, WyCSeg("x",size_t(0))))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="ac") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(1,0, WyCSeg("x",size_t(1))))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axc") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3, WyCSeg("x",size_t(0))))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="") {
    WY_THROW(WyRet() );
  }

/* self-operate
  {
   WyStr a("abcde");
   if((r=a.replace(2,1, WyCSeg(a.data(),a.size())))!=OK) {
     WY_THROW(r);
   }
   if(a!="ababcdede") {
    WY_THROW(WyRet() );
  }
  }
*/

  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,2, WyCSeg("ab",2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3, WyCSeg("xy",2)))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab45xy") {
    WY_THROW(WyRet() );
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,1, WyCSeg(rstr.data(),rstr.size())))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+6) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   size_t t2=tmp.size()-3;
   for(size_t i=3; i<t2; ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if((tmp[t2+0]!='5')||(tmp[t2+1]!='6')||(tmp[t2+2]!='7')) {
     WY_THROW(WyRet() );
   }
  }

  {
   size_t bufsiz;
   tmp.reset(Temp);
   if(tmp!=Temp) {
     WY_THROW(WyRet() );
   }
   bufsiz=tmp._capacity();
   WyStr rstr(size_t(bufsiz+1),char('x'));

   if((r=tmp.replace(3,100, WyCSeg(rstr.data(),rstr.size())))!=OK) {
    WY_THROW(r);
   }
   if(tmp.size()!=rstr.size()+3) {
     WY_THROW(WyRet() );
   }
   if((tmp[0]!='1')||(tmp[1]!='2')||(tmp[2]!='3')) {
     WY_THROW(WyRet() );
   }
   for(size_t i=3; i<tmp.size(); ++i) {
     if(tmp[i]!='x') {
       WY_THROW(WyRet() );
     }
   }
   if(tmp[tmp.size()]!=0) {
     WY_THROW(WyRet() );
   }
  }
 }

 // replace(size_t,size_t,const char*)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  if((r=tmp.replace(0,4,"abc"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="abc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,0,"x"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axbc") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1,"x"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axbc") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3,""))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="c") {
    WY_THROW(WyRet() );
  }

/* self-operate
  {
   WyStr a("abcde");
   if((r=a.replace(2,1,a.c_str()))!=OK) {
     WY_THROW(r);
   }
   if(a!="ababcdede") {
    WY_THROW(WyRet() );
   }
  }
*/

  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,2,"ab"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3,"xy"))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1ab45xy") {
    WY_THROW(WyRet() );
  }
 }

 // replace(size_t,size_t,size_t,ch)
 {
  WyStr tmp;
  if((r=tmp.replace(0,4,0,'x'))!=OK) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
  if((r=tmp.replace(0,4,3,'a'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aaa") {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,1,0,'b'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(2,2,0,'x'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="aa") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(1,0,1,'x'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="axa") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(0,3,0,'t'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="") {
    WY_THROW(WyRet() );
  }

  const WyStr Temp("1234567");
  tmp.reset(Temp);
  if(tmp!=Temp) {
    WY_THROW(WyRet() );
  }

  if((r=tmp.replace(1,2,2,'b'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1bb4567") {
    WY_THROW(WyRet() );
  }
  if((r=tmp.replace(5,3,2,'y'))!=OK) {
    WY_THROW(r);
  }
  if(tmp!="1bb45yy") {
    WY_THROW(WyRet() );
  }
 }

 // copyto(char*,size_t,size_t&,size_t)
 {
  WyStr tmp;
  char buf[8];
  size_t res;
  buf[0]='x';
  if((r=tmp.copyto(&res,buf,sizeof(buf),0))!=OK) {
    WY_THROW(r);
  }
  if(res!=0) {
    WY_THROW(WyRet() );
  }
  if(buf[0]!='x') {
    WY_THROW(WyRet() );
  }

  WyStr tmp2("abcde");
  buf[0]='x';
  if((r=tmp2.copyto(&res,buf,sizeof(buf),3))!=OK) {
    WY_THROW(r);
  }
  if(res!=2) {
    WY_THROW(WyRet() );
  }
  if((buf[0]!='d')||(buf[1]!='e')) {
    WY_THROW(WyRet() );
  }
  if((r=tmp2.copyto(0,buf,1,1))!=OK) {
    WY_THROW(r);
  }
  if(buf[0]!='b') {
    WY_THROW(WyRet() );
  }
 }

 // _reserve
 {
  WyStr tmp;
  size_t cap=tmp._capacity();

  CHK_DEFAULT(tmp);
  if((r=tmp.append('f'))!=OK) {
    WY_THROW(r);
  }
  if(cap!=tmp._capacity()) {
    WY_THROW(WyRet() );  // capacity should no have changed
  }
  if(tmp._reserve(cap)!=OK) {
    WY_THROW(WyRet() );  // capacity should no have changed
  }
  if(cap!=tmp._capacity()) {
    WY_THROW(WyRet() );  // capacity should no have changed
  }

  // double tmp's capacity
  if(tmp._reserve(cap+cap)!=OK) {
    WY_THROW(WyRet() );
  }
  if(cap==tmp._capacity()) {
    WY_THROW(WyRet() );
  }

  WyStr(tmp).swap(tmp);
  if(cap!=tmp._capacity()) {
    WY_THROW(WyRet() );
  }

  {
   const char* obuf=tmp.data();
   size_t cap2=tmp._capacity();
   for(size_t i=tmp.size(); i<cap2; ++i) {
     if((r=tmp.append('x'))!=OK) {
       WY_THROW(r);
     }
   }
   if(tmp._capacity()!=cap2) {
     WY_THROW(WyRet() );
   }
   if(tmp.data()!=obuf) {
     WY_THROW(WyRet() );
   }
  }
 }

 // find(size_t*,char)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find(&res,'a')==true) {
    WY_THROW(WyRet() );
  }
 
  const WyStr tmp1("ppooiiuuyy");
  if(tmp1.find(&res,'p')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1.find(&res,'i')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=4) {
    WY_THROW(WyRet() );
  }
  if(tmp1.find(&res,'y')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=8) {
    WY_THROW(WyRet() );
  }
  if(tmp1.find(&res,'#')==true) {
    WY_THROW(WyRet() );
  }
  if(res!=8) {
    WY_THROW(WyRet() );
  }
 }

#ifdef WY_DEBUG
 // chk the result of std::find(char) consistent with the test above
 {
  const std::string tmp;
  size_t res;
  res=100;
  if((res=tmp.find('a'))!=StdNPos) {
    WY_THROW(WyRet() );
  }
 
  const std::string tmp1("ppooiiuuyy");
  if((res=tmp1.find('p'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=0) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.find('i'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=4) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.find('y'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=8) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.find('#'))!=StdNPos) {
    WY_THROW(WyRet() );
  }
 }
#endif

 // rfind(size_t*,char)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.rfind(&res,'a')==true) {
    WY_THROW(WyRet() );
  }
 
  const WyStr tmp1("ppooiiuuyy");
  if(tmp1.rfind(&res,'p')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  if(tmp1.rfind(&res,'i')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=5) {
    WY_THROW(WyRet() );
  }
  if(tmp1.rfind(&res,'y')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=9) {
    WY_THROW(WyRet() );
  }
  if(tmp1.rfind(&res,'#')==true) {
    WY_THROW(WyRet() );
  }
  if(res!=9) {
    WY_THROW(WyRet() );
  }
 }

#ifdef WY_DEBUG
 // std::string chk for rfind(size_t*,char)
 {
  const std::string tmp;
  size_t res;
  res=100;
  if((res=tmp.rfind('a'))!=StdNPos) {
    WY_THROW(WyRet() );
  }
 
  const std::string tmp1("ppooiiuuyy");
  if((res=tmp1.rfind('p'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.rfind('i'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=5) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.rfind('y'))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=9) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.rfind('#'))!=StdNPos) {
    WY_THROW(WyRet() );
  }
 }
#endif

 // find(size_t*,size_t,char)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find(&res,0,'a')==true) {
    WY_THROW(WyRet() );
  }
  if(res!=100) {
    WY_THROW(WyRet() );
  }

  const WyStr tmp1("rstuvwyyz");
  res=100;
  if(tmp1.find(0,0,'s')==false) {
    WY_THROW(WyRet() );
  }
  if(tmp1.find(&res,0,'s')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  
  int i;
  for(i=0; tmp1.find(&res,i,tmp1[tmp1.size()-1]); ++i,++res) {
    if(res!=tmp1.size()-1) {
      WY_THROW(WyRet() );
    }
  };
  if(i!=int(tmp1.size())) {
    WY_THROW(WyRet() );
  }

  for(i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.find(&res,i,'#')) {
      WY_THROW(WyRet() );
    }
  };
 }

#ifdef WY_DEBUG
 // chk find(size_t*,size_t,char), std::string version
 {
  const std::string tmp;
  size_t res;
  res=100;
  if((res=tmp.find('a',0))!=StdNPos) {
    WY_THROW(WyRet() );
  }

  const std::string tmp1("rstuvwyyz");
  res=100;
  if((res=tmp1.find('s',0))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.find('s',0))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  
  int i;
  for(i=0; i<int(tmp1.size()); ++i) {
    if((res=tmp1.find('#',i))!=StdNPos) {
      WY_THROW(WyRet() );
    }
  };
 }
#endif

 // rfind(size_t*,size_t,ch)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.rfind(&res,0,'a')==true) {
    WY_THROW(WyRet() );
  }
  if(res!=100) {
    WY_THROW(WyRet() );
  }

  const WyStr tmp1("rstuvwyyz");
  res=100;
  if(tmp1.rfind(0,100,'s')==false) {
    WY_THROW(WyRet() );
  }
  if(tmp1.rfind(&res,100,'s')==false) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  
  int i;
  for(res=tmp1.size()-1,i=tmp1.size()-1; (i>=0)&&(tmp1.rfind(&res,100,tmp1[res])); --i,--res) {
    if(i!=int(res)) {
      if(int(res)<i) {
        WY_THROW(WyRet() );
      }
      if(tmp1[res]!='y') {   // we have 2 y's
        WY_THROW(WyRet() );
      }
    }
  };
  //if(res>=tmp1.size()) {
  //  WY_THROW(WyRet() );
  //}

  for(res=tmp1.size()-1,i=tmp1.size()-1; (i>=0)&&(tmp1.rfind(&res,res,tmp1[res])); --i,--res) {
    if(i!=int(res)) {
      WY_THROW(WyRet() );
    }
  };
  //if(res>=tmp1.size()) {
  //  WY_THROW(WyRet() );
  //}

  for(i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.rfind(&res,i,'#')) {
      WY_THROW(WyRet() );
    }
  };
 }

#ifdef WY_DEBUG
 // rfind(size_t*,size_t,ch) chk std::string version
 {
  const std::string tmp;
  size_t res;
  res=100;
  if((res=tmp.rfind('a',0))!=StdNPos) {
    WY_THROW(WyRet() );
  }

  const std::string tmp1("rstuvwyyz");
  res=100;
  if((res=tmp1.rfind('s',100))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if((res=tmp1.rfind('s',100))==StdNPos) {
    WY_THROW(WyRet() );
  }
  if(res!=1) {
    WY_THROW(WyRet() );
  }
  
  int i;
  for(i=0; i<int(tmp1.size()); ++i) {
    if((res=tmp1.rfind('#',i))!=StdNPos) {
      WY_THROW(WyRet() );
    }
  };
 }
#endif

 // find(size_t*,size_t,const char*,size_t)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find(&res,0, WyCSeg("a",size_t(0)))==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.find(&res,0, WyCSeg("a",1))==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.find(0,0, WyCSeg("wy",2))==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.find(&res,0, WyCSeg("wy",2))==false) {
    WY_THROW( WyRet() );
  }
  if(res!=4) {
    WY_THROW( WyRet() );
  }
  
  int i;
  for(i=0; tmp1.find(&res,i,WyCSeg(&tmp1[tmp1.size()-2],2)); ++i,++res) {
    if(res!=tmp1.size()-2) {
      WY_THROW( WyRet() );
    }
  };
  if(i!=int(tmp1.size())-1) {
    WY_THROW( WyRet() );
  }

  for(i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.find(&res,i, WyCSeg("#",1))) {
      WY_THROW( WyRet() );
    }
  };
 }

 // find(size_t*,size_t,const char*)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find(&res,0,"")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.find(&res,0,"a")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.find(0,0,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.find(&res,0,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(res!=4) {
    WY_THROW( WyRet() );
  }
  
  for(int i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.find(&res,i,"#")) {
      WY_THROW( WyRet() );
    }
  };
 }

 // find(size_t*,const WyCSeg&)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find(&res,"")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.find(&res,"a")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.find(0,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.find(&res,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(res!=4) {
    WY_THROW( WyRet() );
  }
 }

 // rfind(size_t*,size_t,const char*,size_t)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  CHK_DEFAULT(tmp);
  if(tmp.rfind(&res,0, WyCSeg("a",size_t(0)))==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind(&res,0, WyCSeg("a",1))==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.rfind(0,100, WyCSeg("wy",2))==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.rfind(&res,100, WyCSeg("wy",2))==false) {
    WY_THROW( WyRet() );
  }
  if(res!=6) {
    WY_THROW( WyRet() );
  }
  
  for(int i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.rfind(&res,i, WyCSeg("#",1))) {
      WY_THROW( WyRet() );
    }
  };
 }

 // rfind(size_t*,size_t,const char*)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.rfind(&res,0,"")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind(&res,0,"a")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.rfind(0,100,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.rfind(&res,100,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(res!=6) {
    WY_THROW( WyRet() );
  }
  
  for(int i=0; i<int(tmp1.size()); ++i) {
    if(tmp1.rfind(&res,i, WyCSeg("#",1))) {
      WY_THROW( WyRet() );
    }
  };
 }

 // rfind(size_t*,const WyCSeg&)
 {
  const WyStr tmp;
  size_t res;
  res=100;
  if(tmp.rfind(&res,"")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind(&res,"a")==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  const WyStr tmp1("rstuwywyyz");
  res=100;
  if(tmp1.rfind(0,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.rfind(&res,"wy")==false) {
    WY_THROW( WyRet() );
  }
  if(res!=6) {
    WY_THROW( WyRet() );
  }
 }

 // r/find_not(size_t*,size_t,char)
 {
  WyStr tmp;
  size_t res;
  res=100;
  if(tmp.find_not(&res,0,'a')==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind_not(&res,0,'a')==true) {
    WY_THROW( WyRet() );
  }
  if(res!=100) {
    WY_THROW( WyRet() );
  }

  tmp="0X0X000";
  if(tmp.find_not(&res,0,'0')==false) {
    WY_THROW( WyRet() );
  }
  if(res!=1) {
    WY_THROW( WyRet() );
  }
  if(tmp.find_not(&res,res+1,'0')==false) {
    WY_THROW( WyRet() );
  }
  if(res!=3) {
    WY_THROW( WyRet() );
  }
  if(tmp.find_not(&res,res+1,'0')==true) {
    WY_THROW( WyRet() );
  }
  if(res!=3) {
    WY_THROW( WyRet() );
  }

  if(tmp.rfind_not(&res,tmp.size()+1,'0')==false) {
    WY_THROW( WyRet() );
  }
  if(res!=3) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind_not(&res,res-1,'0')==false) {
    WY_THROW( WyRet() );
  }
  if(res!=1) {
    WY_THROW( WyRet() );
  }
  if(tmp.rfind_not(&res,res-1,'0')==true) {
    WY_THROW( WyRet() );
  }
  if(res!=1) {
    WY_THROW( WyRet() );
  }
 }

 // substr() modified to cseg()
 {
  WyStr tmp_str("xy564");
  const WyCSeg cseg(tmp_str.cseg());
  if(cseg.begin()!=tmp_str.data()) {
    WY_THROW(WyRet() );
  }
  if(cseg.size()!=tmp_str.size()) {
    WY_THROW(WyRet() );
  }
 }

 // substr(..) modified to cseg(..)
 {
  WyStr tmp;
  CHK_DEFAULT(tmp.cseg(0));
  CHK_DEFAULT(tmp.cseg(0,0));
  CHK_DEFAULT(tmp.cseg(0,2));

  WyStr tmp1("abcd");
  if( WyStr(tmp1.cseg(2))!="cd") {
    WY_THROW(WyRet() );
  }
  if( WyStr(tmp1.cseg(2,5))!="cd") {
    WY_THROW(WyRet() );
  }
  if( WyStr(tmp1.cseg(0))!="abcd") {
    WY_THROW(WyRet() );
  }
  if( WyStr(tmp1.cseg(0,5))!="abcd") {
    WY_THROW(WyRet() );
  }
 }

 // cseg
 {
  WyStr tmp;
  WyCSeg cseg( tmp.cseg(0) );
  if(cseg.size()!=0) {
    WY_THROW(WyRet() );
  }
  if(cseg.begin()!=tmp.data()) {
    WY_THROW(WyRet() );
  }

  const WyStr tmp1("bcd");
  cseg.reset(tmp1.cseg(1));
  if(cseg.size()!=tmp1.size()-1) {
    WY_THROW(WyRet() );
  }
  if(cseg.begin()!=tmp1.data()+1) {
    WY_THROW(WyRet() );
  }

  cseg.reset(tmp1.cseg());
  if(cseg.size()!=tmp1.size()) {
    WY_THROW(WyRet() );
  }
  if(cseg.begin()!=tmp1.data()) {
    WY_THROW(WyRet() );
  }

  cseg.reset(tmp1.cseg(1,1));
  if(cseg.size()!=1) {
    WY_THROW(WyRet() );
  }
  if(cseg.begin()!=tmp1.data()+1) {
    WY_THROW(WyRet() );
  }

  cseg.reset(tmp1.cseg(1,100));
  if(cseg.size()!=tmp1.size()-1) {
    WY_THROW(WyRet() );
  }
  if(cseg.begin()!=tmp1.data()+1) {
    WY_THROW(WyRet() );
  }
 }

 // strcmp and ==
 {
  const WyStr tmp0;
  const WyStr tmp1a("qwert");
  const WyStr tmp1b("qweru");
  const WyStr tmp2a("qwerty");
  const WyStr tmp2b("qwertu");
  if(tmp0!="") {
    WY_THROW(WyRet() );
  }
  if(tmp0.strcmp(tmp0)!=0) {
    WY_THROW(WyRet() );
  }
  if(tmp0.strcmp(tmp1a)>=0) {
    WY_THROW(WyRet() );
  }

  if(tmp1a.strcmp(tmp0)<=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp1a)!=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp1b)>=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp2a)>=0) {
    WY_THROW(WyRet() );
  }

  if(tmp1a.strcmp(tmp0.c_str())<=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp1a.c_str())!=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp1b.c_str())>=0) {
    WY_THROW(WyRet() );
  }
  if(tmp1a.strcmp(tmp2a.c_str())>=0) {
    WY_THROW(WyRet() );
  }

  if(tmp2a.strcmp(tmp0)<=0) {
    WY_THROW(WyRet() );
  }
  if(tmp2a.strcmp(tmp2b)==0) {
    WY_THROW(WyRet() );
  }
  if(tmp2a.strcmp(tmp1b)<=0) {
    WY_THROW(WyRet() );
  }
  if(tmp2a.strcmp(tmp2a)!=0) {
    WY_THROW(WyRet() );
  }

  // Wy::_strcmp
  if(Wy::_strcmp(tmp1a,tmp0)<=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a,tmp1a)!=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),tmp0)<=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),tmp1a)>=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),tmp0.c_str())<=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),tmp1a.c_str())>=0) {
    WY_THROW(WyRet() );
  }

  if(Wy::_strcmp(tmp1a.cseg(1,2),tmp0)<=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2).begin(),tmp1a)>=0) {
    WY_THROW(WyRet() );
  }

  if(Wy::_strcmp(tmp1a.cseg(1,2),WyCSeg(tmp0.c_str()+1,1))<=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),WyCSeg(tmp1a.c_str()+1,3))>=0) {
    WY_THROW(WyRet() );
  }
  if(Wy::_strcmp(tmp1a.cseg(1,2),WyCSeg(tmp1a.c_str()+1,2))!=0) {
    WY_THROW(WyRet() );
  }

  if(tmp1a==(const char*)tmp1a.c_str()) {
  } else {
    WY_THROW(WyRet() );
  }
  if(tmp1a!=(const char*)tmp1a.c_str()) {
    WY_THROW(WyRet() );
  }
 }

 // _reserve(size_t)
 {
   const char* STR="aaa";
   WyStr str(STR);
   if(str!=STR) {
     WY_THROW( WyRet() );
   }
   if(str.size()!=std::strlen(STR)) {
     WY_THROW( WyRet() );
   }
   if(str._capacity()<WyStr::min_capacity()) {
     WY_THROW( WyRet() );
   }

   if(str._reserve(1)!=OK) {
     WY_THROW( WyRet() );
   }
   if(str!=STR) {
     WY_THROW( WyRet() );
   }
   if(str.size()!=std::strlen(STR)) {
     WY_THROW( WyRet() );
   }
   if(str._capacity()<WyStr::min_capacity()) {
     WY_THROW( WyRet() );
   }

   if(str._reserve(10)!=OK) {
     WY_THROW( WyRet() );
   }
   if(str!=STR) {
     WY_THROW( WyRet() );
   }
   if(str.size()!=std::strlen(STR)) {
     WY_THROW( WyRet() );
   }
   if(str._capacity()<WyStr::min_capacity()) {
     WY_THROW( WyRet() );
   }

   if(str._reserve(100000)!=OK) {
     WY_THROW( WyRet() );
   }
   if(str!=STR) {
     WY_THROW( WyRet() );
   }
   if(str.size()!=std::strlen(STR)) {
     WY_THROW( WyRet() );
   }
   if(str._capacity()<100000) {
     WY_THROW( WyRet() );
   }
 
   // just chk is_ok
   //if(str._reserve(100000).is_ok()) {
   //  WY_THROW( WyRet() );
   //}
 }

 // operator +
 {
  const char* cstr="bt";
  const char  ch('x');
  const WyStr str("gfh");

  if((str+str)!=WyStr("gfhgfh")) {
    WY_THROW(WyRet() );
  }
  if((str+str.cseg())!=WyStr("gfhgfh")) {
    WY_THROW(WyRet() );
  }
  if((str+cstr)!=WyStr("gfhbt")) {
    WY_THROW(WyRet() );
  }

  if((WyCSeg(cstr)+str)!=WyStr("btgfh")) {
    WY_THROW(WyRet() );
  }

  if((cstr+str)!=WyStr("btgfh")) {
    WY_THROW(WyRet() );
  }
  if((cstr+str.cseg())!=WyStr("btgfh")) {
    WY_THROW(WyRet() );
  }

  if((str+ch)!=WyStr("gfhx")) {
    WY_THROW(WyRet() );
  }
  if((ch+str)!=WyStr("xgfh")) {
    WY_THROW(WyRet() );
  }
 }

 // aux: ==(const WyCSeg&,const WyStr&)
 {
  const WyCSeg str1("bt"),str2("u");
  if((const WyCSeg&)str1==WyStr(str1)) {
  } else {
    WY_THROW( WyRet() );
  }
  if((const WyCSeg&)str1!=WyStr(str1)) {
    WY_THROW( WyRet() );
  }
  if((const WyCSeg&)str1==WyStr(str2)) {
    WY_THROW( WyRet() );
  }
  if((const WyCSeg&)str1!=WyStr(str2)) {
  } else {
    WY_THROW( WyRet() );
  }
 }

 // aux: ==(const char*,const WyStr&)
 {
  const char *str1="bt",*str2="u";
  if(str1==WyStr(str1)) {
  } else {
    WY_THROW( WyRet() );
  }
  if(str1!=WyStr(str1)) {
    WY_THROW( WyRet() );
  }
  if(str1==WyStr(str2)) {
    WY_THROW( WyRet() );
  }
  if(str1!=WyStr(str2)) {
  } else {
    WY_THROW( WyRet() );
  }
 }

 // chk _setsize(size_t)
 {
  WyStr tmp;
  tmp._setsize(0);
  CHK_DEFAULT(tmp);

  const char* TStr="abc";
  tmp=TStr;
  if(tmp!=TStr) {
    WY_THROW( WyRet() );
  }
  tmp._setsize(0);
  CHK_DEFAULT(tmp);

  tmp._setsize(1);
  if(tmp!=WyCSeg("\0",1)) {
    WY_THROW( WyRet() );
  }
  tmp._setsize(3);
  if(tmp!=WyCSeg("\0\0c",3)) {
    WY_THROW( WyRet() );
  }

  {
    WyStr tmp;
    if((r=tmp._setsize(WyStr::max_capacity()+1))!=Wym_EFBIG) {
      WY_THROW(r);
    }
  }
 }

 // chk move ctor
 {
  char buf[2*sizeof(WyStr)];
  ::memset(buf,0,sizeof(buf));
  WyStr*p=reinterpret_cast<WyStr*>(buf);
  new(p) WyStr(WyCSeg("pm",2));
  new(p+1) WyStr(*p,Wy::ByMove);
  if((p+1)->strcmp("pm")!=0) {
    (p+1)->~WyStr();
    WY_THROW( WyRet() );
  }
  (p+1)->~WyStr();
 }
};

static void t1_error(void)
{
 WyRet r;

 // chk error WyStr(const WyStr&,size_t)
 try {
   WyStr tmp("123");
   WyStr tmp1(tmp.cseg(4));
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 // chk error WyStr(const WyStr&,size_t,size_t)
 try {
   WyStr tmp("123");
   WyStr tmp1(tmp.cseg(4,1));
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 // chk error WyStr(const WyCSeg&)
 try {
   WyStr tmp( WyCSeg(0,1) );
   WY_THROW( WyRet() );
 }
 catch(const WyCSeg::Reply& e) {
   if(e!=Wym_EFAULT) {
     throw;
   }
   // FALLTHOUGH
 };

 // chk error at(size_t)
 try {
   WyStr tmp;
   tmp.at(tmp.size()+1);
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 const WyStr NZSTR("x");   // non zero string for test. as operation may say
                           // ok if argument str is zero size
 WyStr tmp("khio");

 // chk error reset(const WyStr&,size_t)
 if((r=tmp.reset(tmp))!=Wym_ELOOP) {
   WY_THROW(r);
 }

 try {
   tmp.reset(tmp.cseg(tmp.size()+1));
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALL_THROUGH
 };

 try {
   tmp.reset(tmp.cseg(tmp.size()+1,1));
   WY_THROW(r);
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALL_THROUGH
 };

 if((r=tmp.reset(tmp.cseg(1,2)))!=Wym_ELOOP) {
   WY_THROW(r);
 }
 if((r=tmp.reset(tmp.cseg(2,1)))!=OK) {
   // This is imp. feature, src/des on the same buffer though
   WY_THROW(r);
 }
 if((r=tmp.reset(tmp.cseg(tmp.size()-1,1)))!=Wym_ELOOP) {
   WY_THROW(r);
 }

 try {
   tmp.append(tmp.cseg(tmp.size()+1));
   WY_THROW(r);
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALL_THROUGH
 };

 // check error operator =
 {
  try {
    tmp=tmp.cseg();
  }
  catch(const WyStr::Reply& e) {
    if(e!=Wym_ELOOP) {
      throw;
    }
    // FALL_THROUGH
  };

  try {
    tmp=tmp;
  }
  catch(const WyStr::Reply& e) {
    if(e!=Wym_ELOOP) {
      throw;
    }
    // FALL_THROUGH
  };
 }

 // check error append(..)
 try {
   tmp.append(tmp.cseg(tmp.size()+1,1));
   WY_THROW(r);
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALL_THROUGH
 };

 // check error insert(..)
 if((r=tmp.insert(tmp.size()+1,NZSTR))!=Wym_EINVAL) {
   WY_THROW(r);
 }

 if((r=tmp.insert(tmp.size()+1,1,'a'))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 {
  WyStr tmp4("abcd");
  if((r=tmp4.insert(0,tmp4))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.insert(tmp4.size()-1,tmp4))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.insert(tmp4.size(),tmp4))!=OK) {
    WY_THROW(r);
  }
  if(tmp4!="abcdabcd") {
    WY_THROW( WyRet() );
  }
 }

 // check error erase(..)
 if((r=tmp.erase(tmp.size()+1))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 if((r=tmp.erase(tmp.size()+1,1))!=Wym_EINVAL) {
   WY_THROW( WyRet() );
 }

 // chk error replace(..)
 if((r=tmp.replace(tmp.size()+1,1,NZSTR))!=Wym_EINVAL) {
   WY_THROW( WyRet() );
 }
 if((r=tmp.replace(tmp.size()+1,1,NZSTR.cseg(0)))!=Wym_EINVAL) {
   WY_THROW( WyRet() );
 }

 if((r=tmp.replace(tmp.size()+1,1,NZSTR.cseg(0,1)))!=Wym_EINVAL) {
   WY_THROW( WyRet() );
 }

 if((r=tmp.replace(tmp.size()+1,1,WyCSeg("",1)))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 if((r=tmp.replace(tmp.size()+1,1,1,'x'))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 {
  // chk error
  WyStr tmp4("13579");
  WyStr TMP4;
  if((r=tmp4.replace(0,1,tmp4))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.replace(tmp4.size()-1,1,tmp4))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.replace(1,3,tmp4))!=Wym_ELOOP) {
    WY_THROW(r);
  }

  TMP4=tmp4;
  if((r=tmp4.replace(tmp4.size(),0,tmp4))!=OK) {
    if(tmp4!=TMP4) {
      WY_THROW( WyRet() ); // failure postcondition
    }
    if(r!=Wym_ELOOP) {
      WY_THROW(r);  // protocol violated
    }
  } else {
    if(tmp4!=TMP4+TMP4) {
      WY_THROW( WyRet() );
    }
  }

  if((r=tmp4.replace(0,1,tmp4.cseg()))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.replace(tmp4.size()-1,1,tmp4.cseg()))!=Wym_ELOOP) {
    WY_THROW(r);
  }
  if((r=tmp4.replace(1,3,tmp4.cseg()))!=Wym_ELOOP) {
    WY_THROW(r);
  }

  TMP4=tmp4;
  if((r=tmp4.replace(tmp4.size(),0,tmp4.cseg()))!=OK) {
    if(tmp4!=TMP4) {
      WY_THROW( WyRet() ); // failure postcondition
    }
    if(r!=Wym_ELOOP) {
      WY_THROW(r);  // protocol violated
    }
  } else {
    if(tmp4!=TMP4+TMP4) {
      WY_THROW( WyRet() );
    }
  }

  TMP4=tmp4;
  if((r=tmp4.replace(tmp4.size()-1,1,WyCSeg(tmp4.data()+1,3)))!=OK) {
    // protocol not violated, though
    if(tmp4!=TMP4) {
      WY_THROW( WyRet() ); // failure postcondition
    }
    if(r!=Wym_ELOOP) {
      WY_THROW(r);  // protocol violated
    }
  } else {
    TMP4.resize(TMP4.size()-1);
    TMP4+=tmp4.cseg(1,3);
    if(tmp4!=TMP4) {
      WY_THROW( WyRet() );
    }
  }
 };
 
 // chk error cseg
 try {
   WyStr tt( tmp.cseg(tmp.size()+1) );
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 try {
   WyStr tt( tmp.cseg(tmp.size()+1,1) );
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 // chk error subseg
 try {
   WyCSeg tt( tmp.cseg(tmp.size()+1) );
   tt.size();          // supress unused message of compiler
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 try {
   WyCSeg tt( tmp.cseg(tmp.size()+1,1) );
   tt.size();          // supress unused message of compiler
   WY_THROW( WyRet() );
 }
 catch(const WyStr::Reply& e) {
   if(e!=Wym_EINVAL) {
     throw;
   }
   // FALLTHOUGH
 };

 // chk error _copyto
 {
   char buf[4];
   if((r=tmp.copyto(0,buf,sizeof(buf),tmp.size()+1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   tmp="24680";
   char *tptr=(char*)tmp.data();
   size_t res=999;
   if((r=tmp.copyto(&res,tptr,1,2))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=tmp.copyto(&res,tptr,1,2))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if(res!=999) {
     WY_THROW( WyRet() );
   }
 }

 // chk error operator==(const char*)
 {
   try {
     WyStr tmp;
     if(tmp==(const char*)0) {
       tmp.reset();
     }
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error operator!=(const char*)
 {
   try {
     WyStr tmp;
     if(tmp!=(const char*)0) {
       tmp.reset();
     }
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error strcmp(const char*)
 {
   try {
     WyStr tmp;
     if(tmp.strcmp((const char*)0)) {
       tmp.reset();
     }
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error operator +(const WyStr&,const WyCSeg&)
 {
   try {
     WyStr tmp;
     tmp= tmp+WyCSeg("",WyStr::max_capacity()+1);
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EFBIG) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error operator +(const WyCSeg&,const WyStr&)
 {
   try {
     WyStr tmp;
     tmp= WyCSeg("",WyStr::max_capacity()+1)+tmp;
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EFBIG) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk instance(error) "const char*+WyStr"
 {
   try {
     WyStr tmp;
     tmp= (const char*)0+tmp;
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk instance(error) "WyStr+const char*"
 {
   try {
     WyStr tmp;
     tmp= tmp+(const char*)0;
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error operator==(const char*,const WyStr&)
 {
   try {
     WyStr tmp;
     if((const char*)0==tmp) {
       tmp.reset();
     }
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error operator!=(const char*,const WyStr&)
 {
   try {
     WyStr tmp;
     if((const char*)0!=tmp) {
       tmp.reset();
     }
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // chk error Wy::_strcmp(const WyStr&,const char*)
 {
   try {
     WyStr tmp;
     Wy::_strcmp(tmp,(const char*)0);
     tmp.reset();
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }
 // chk error Wy::_strcmp(const char*,const WyStr&)
 {
   try {
     WyStr tmp;
     Wy::_strcmp((const char*)0,tmp);
     tmp.reset();
     WY_THROW( WyRet() );
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALLTHROUGH
   };
 }
};

static void t2(void)
{
};

template <typename NumType>
static void t_mkstr_integer(void)
{
 struct ChkSmp {
   const char* nstr;
   NumType num;
   int     radix;
   void verify(void) const {
     WyRet r;
     WyStr str;
     try {
       WyStr tmp;
       if((r=Wy::_mkstr(str,num,radix))!=OK) {
         WY_THROW(r);
       }
       if(str!=nstr) {
         WY_THROW( WyRet() );
       }
       if(radix==10) {
         if((r=Wy::_mkstr(tmp,num))!=OK) {
           WY_THROW(r);
         }
         if(tmp!=nstr) {
           WY_THROW( WyRet() );
         }
         if((r=Wy::_mkstr(tmp,num))!=OK) {
           WY_THROW(r);
         }
         if(tmp!=nstr) {
           WY_THROW( WyRet() );
         }

         if(tmp!=Wy::wrd(num)) {
           WY_THROW( WyRet() );
         }

         if((r=Wy::_catstr(tmp,num))!=OK) {
           WY_THROW(r);
         }
         if(tmp!=WyStr(nstr)+nstr) {
           WY_THROW( WyRet() );
         }
       }
       tmp=str;
       if((r=Wy::_catstr(str,num,radix))!=OK) {
         WY_THROW(r);
       }
       if(str!=tmp+tmp) {
         WY_THROW( WyRet() );
       }

       if((r=Wy::_mkstr(str,num,radix,0))!=OK) {
         WY_THROW(r);
       }
       if(str!=nstr) {
         WY_THROW( WyRet() );
       }
       tmp=str;
       if((r=Wy::_catstr(str,num,radix,0))!=OK) {
         WY_THROW(r);
       }
       if(str!=tmp+tmp) {
         WY_THROW( WyRet() );
       }

       str=Wy::wrd(num,radix);
       if(str!=nstr) {
         WY_THROW( WyRet() );
       }
       str=Wy::wrd(num,radix,0);
       if(str!=nstr) {
         WY_THROW( WyRet() );
       }
     }
     catch(...) {
       std::cerr << "ChkSmp{\"" << nstr << "\"," <<  (long long)num
                 << "," << radix << "}  checked=" << str.c_str() << std::endl;
       throw;
     };
   }
 };

 if(std::numeric_limits<NumType>::is_signed) {
   const ChkSmp chk_tab[]={
     {"0",NumType(),10},
     {"1",NumType(1),2}, {"-1",NumType(-1),2},
     {"1",NumType(1),8}, {"-1",NumType(-1),8},
     {"1",NumType(1),10}, {"-1",NumType(-1),10},
     {"1",NumType(1),16}, {"-1",NumType(-1),16},
     {"1",NumType(1),35}, {"-1",NumType(-1),35},
     {"10",NumType(2),2}, {"-10",NumType(-2),2},
     {"2",NumType(2),8}, {"-2",NumType(-2),8},
     {"2",NumType(2),10}, {"-2",NumType(-2),10},
     {"2",NumType(2),16}, {"-2",NumType(-2),16},
     {"2",NumType(2),35}, {"-2",NumType(-2),35},
  
     {"1111011",NumType(123),2}, {"-1111011",NumType(-123),2},
     {"173",NumType(123),8}, {"-173",NumType(-123),8},
     {"123",NumType(123),10}, {"-123",NumType(-123),10},
     {"7B",NumType(123),16}, {"-7B",NumType(-123),16},
     {"3F",NumType(123),36}, {"-3F",NumType(-123),36},
   };
   for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
     chk_tab[i].verify();
   }
 } else {
   const ChkSmp chk_tab[]={
     {"0",NumType(),10},
     {"1",NumType(1),2},
     {"1",NumType(1),8},
     {"1",NumType(1),10},
     {"1",NumType(1),16},
     {"1",NumType(1),35},
     {"10",NumType(2),2},
     {"2",NumType(2),8},
     {"2",NumType(2),10},
     {"2",NumType(2),16},
     {"2",NumType(2),35},
  
     {"1111011",NumType(123),2},
     {"173",NumType(123),8},
     {"123",NumType(123),10},
     {"7B",NumType(123),16},
     {"3F",NumType(123),36}
   };
   for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
     chk_tab[i].verify();
   }
 }

 // chk error
 {
   WyRet r;
   WyStr str;
   const NumType num(0);
   if((r=Wy::_mkstr(str,num,10,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_mkstr(str,num,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_catstr(str,num,10,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_catstr(str,num,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }

   try {
     Wy::wrd(num,(int)-1);
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EINVAL) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };

   try {
     Wy::wrd(num,(int)-1,(size_t)0);
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EINVAL) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };
 }
};

template <typename NumType>
static void t_mkstr_float(void)
{
 struct ChkSmp {
   const char* nstr;
   const NumType num;
   const int radix;
   const size_t  frdig;
   void verify(void) const {
     WyRet r;
     WyStr str;
     try {
       WyStr tmp;
       if((r=Wy::_mkstr(str,num,radix,frdig))!=OK) {
         WY_THROW(r);
       }
       if(str!=nstr) {
         WY_THROW( WyRet() );
       }

       tmp=str;
       if((r=Wy::_catstr(str,num,radix,frdig))!=OK) {
         WY_THROW(r);
       }
       if(str!=tmp+tmp) {
         WY_THROW( WyRet() );
       }
     
       tmp=Wy::wrd(num,radix,frdig);
       if(tmp!=nstr) {
         WY_THROW( WyRet() );
       }
     }
     catch(...) {
       std::cerr << "ChkSmp{\"" << nstr << "\"," << num
                 << "," << radix << "," << frdig
                 << "}  checked=" << str.c_str() << std::endl;
       throw;
     };
   }
 } const chk_tab[]={
   {"0",NumType(),10,0},{"0.0",NumType(),10,1}, {"0.00",NumType(),10,2},
   {"0",NumType(0.1),10,0},{"0.1",NumType(0.1),10,1},
                            {"-0.1",NumType(-0.1),10,1},
   {"0",NumType(-0.2),10,0},{"-0.2",NumType(-0.2),10,1},
                            {"-0.20",NumType(-0.2),10,2},
   {"10",NumType(10.0),10,0},{"10.0",NumType(10.0),10,1},
                             {"-10.0",NumType(-10.0),10,1},
   {"20",NumType(20.0),10,0},{"20.0",NumType(20.0),10,1},
                            {"-20.0",NumType(-20.0),10,1},
   {"12345",NumType(12345.0),10,0},
   {"12345.0000",NumType(12345.0),10,4},
   {"-123",NumType(-123.45),10,0},
   {"-123.43",NumType(-123.43),10,2},

   {"0e+0",NumType(),0,0},{"0.0e+0",NumType(),0,1},
                           {"0.00e+0",NumType(),0,2},
   {"1e+1",NumType(10.0),0,0},{"1.0e+1",NumType(10.0),0,1},
                           {"-1.0e+1",NumType(-10.0),0,1},
   {"1e-1",NumType(0.1),0,0},{"1.0e-1",NumType(0.1),0,1},
                           {"-1.0e-1",NumType(-0.1),0,1},
   {"2e+1",NumType(20.0),0,0},{"2.0e+1",NumType(20.0),0,1},
                           {"-2.0e+1",NumType(-20.0),0,1},
   {"2e-1",NumType(0.2),0,0},{"2.0e-1",NumType(0.2),0,1},
                           {"-2.0e-1",NumType(-0.2),0,1},
   {"-2e-1",NumType(-0.2),0,0},{"-2.0e-1",NumType(-0.2),0,1},
                           {"-2.00e-1",NumType(-0.2),0,2},
   {"4e-7",NumType(4.4e-7),0,0},{"4.4e-7",NumType(4.4e-7),0,1},
                           {"4.40e-7",NumType(4.4e-7),0,2},
   {"-4e-7",NumType(-4.4e-7),0,0},{"-4.4e-7",NumType(-4.4e-7),0,1},
                           {"-4.40e-7",NumType(-4.4e-7),0,2},
   {"1.2345e+4",NumType(12345.0),0,4},
   {"1.23450e+4",NumType(12345.0),0,5},
   {"1.234e+1",NumType(12.345),0,3},
   {"1.2345e-1",NumType(0.12345),0,4},
   {"1e-1",NumType(0.12345),0,0},
   {"10000000000",NumType(1e10),10,0},
 };
 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
   chk_tab[i].verify();
 }

 // chk inf/nan
 {
   WyRet r;
   WyStr str;
   NumType num(66e999);
   if((r=Wy::_mkstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="inf") {
     WY_THROW( WyRet() );
   }
   if((r=Wy::_catstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="infinf") {
     WY_THROW( WyRet() );
   }
   num=-66e999;
   if((r=Wy::_mkstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="-inf") {
     WY_THROW( WyRet() );
   }
   if((r=Wy::_catstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="-inf-inf") {
     WY_THROW( WyRet() );
   }

   num=std::sqrt(-1.0);
   if((r=Wy::_mkstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="nan") {
     WY_THROW( WyRet() );
   }
   if((r=Wy::_catstr(str,num,10,1))!=OK) {
     WY_THROW(r);
   }
   if(str!="nannan") {
     WY_THROW( WyRet() );
   }
 }

 // chk long string
 {
   WyRet r;
   WyStr str;
   NumType num;
    
   if(std::numeric_limits<NumType>::max()>=std::numeric_limits<double>::max()) {
     num=1e23;
     if((r=Wy::_mkstr(str,num,10,1))!=OK) {
       WY_THROW(r);
     }
     if(str!="100000000000000000000000.0") {
       std::cout << str.c_str() << '\n';
       WY_THROW( WyRet() );
     }
     num=-1e23;
     if((r=Wy::_mkstr(str,num,10,0))!=OK) {
       WY_THROW(r);
     }
     if(str!="-100000000000000000000000") {
       WY_THROW( WyRet() );
     }
     num=1e23;
     if((r=Wy::_mkstr(str,num,0,1))!=OK) {
       WY_THROW(r);
     }
     if(str!="1.0e+23") {
       std::cout << str.c_str() << '\n';
       WY_THROW( WyRet() );
     }
     num=-1e23;
     if((r=Wy::_mkstr(str,num,0,0))!=OK) {
       WY_THROW(r);
     }
     if(str!="-1e+23") {
       WY_THROW( WyRet() );
     }

     if((r=Wy::_catstr(str,num,10,1))!=OK) {
       WY_THROW(r);
     }
     if(str!="-1e+23-100000000000000000000000.0") {
       WY_THROW( WyRet() );
     }
   }
 }

 // chk wd(NumType,int)
 {
  WyStr str( Wy::wrd((NumType)4.5,10) );
  if(str!="4.500000") {
    WY_THROW( WyRet() );
  }
 }

 // chk error
 {
   WyRet r;
   WyStr str;
   const NumType num(0);
   if((r=Wy::_mkstr(str,num,9,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_mkstr(str,num,9))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_catstr(str,num,9,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=Wy::_catstr(str,num,9))!=Wym_EINVAL) {
     WY_THROW(r);
   }

   try {
     Wy::wrd(num,(int)-1);
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EINVAL) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };

   try {
     Wy::wrd(num,(int)-1,(size_t)0);
     WY_THROW( WyRet() );
   }
   catch(const WyStr::Reply& e) {
     if(e!=Wym_EINVAL) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };

 }
};

static void t_mkstr(void)
{
 t_mkstr_integer<char>();
 t_mkstr_integer<signed char>();
 t_mkstr_integer<signed short>();
 t_mkstr_integer<signed int>();
 t_mkstr_integer<signed long>();
 t_mkstr_integer<signed long long>();
 t_mkstr_integer<unsigned char>();
 t_mkstr_integer<unsigned short>();
 t_mkstr_integer<unsigned int>();
 t_mkstr_integer<unsigned long>();
 t_mkstr_integer<unsigned long long>();

 t_mkstr_float<float>();
 t_mkstr_float<double>();
};

static void t_share(void)
{
 {
  WyStr t1("xyz");
  WyStr t2(t1);

  t1.reset();
  if(t2!="xyz") {
    WY_THROW(WyRet() );
  }
 }

 // general problem#1
 {
  WyStr t1("xyz");
  const WyStr& rt1(t1);
  WyStr t2(rt1);

  t2[0]='X';
  if(t1!="xyz") {
    WY_THROW(WyRet() );
  }
  if(t2!="Xyz") {
    WY_THROW(WyRet() );
  }
 }

 // general problem#2
 {
  WyStr t1("xyz");
  char* tp1=&t1[0];
  WyStr t2(t1);

  *tp1='X';
  if(t1!="Xyz") {
    WY_THROW(WyRet() );
  }
  if(t2!="xyz") {
    WY_THROW(WyRet() );
  }
 }
};

class CHK_STR {
    WyStr _s1;
    std::string _s2;
  public:
    CHK_STR() 
    : _s1(),_s2() {
      CHK_DEFAULT(_s1);
    };
    CHK_STR(const CHK_STR& src)
    : _s1(src._s1),_s2(src._s2) {
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };
    CHK_STR(const char* cstr)
    : _s1(cstr),_s2(cstr) {
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };
    CHK_STR(size_t num, char ch)
    : _s1(num,ch),_s2(num,ch) {
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };
    const char* c_str(void) const 
       { return _s1.c_str(); };
    const WyStr& wystr(void) const 
       { return _s1; };
    size_t size(void) const
       { return _s1.size(); };

    void reset(void) {
      _s1.reset();
      _s2.clear();
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void reset(const char* cstr) {
      _s1.reset(cstr);
      _s2.assign(cstr);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void reset(const WyStr& str) {
      _s1.reset(str);
      _s2=str.c_str();
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    }

    void reset(size_t num, char ch) {
      _s1.reset(num,ch);
      _s2.assign(num,ch);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    }

    void append(char ch) {
      _s1.append(ch);
      _s2+=ch;
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void append(const char* cstr) {
      _s1.append(cstr);
      _s2.append(cstr);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void append(const WyStr& str) {
      _s1.append(str);
      _s2.append(str.c_str());
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    }

    void append(size_t num, char ch) {
      _s1.append(num,ch);
      _s2.append(num,ch);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    }

    void insert(size_t idx, const char* cstr) {
      _s1.insert(idx,cstr);
      _s2.insert(idx,cstr);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void insert(size_t idx, const WyStr& str) {
      _s1.insert(idx,str);
      _s2.insert(idx,str.c_str());
      if(_s2.size()!=_s1.size()) {
        std::cout << _s1.c_str() << std::endl;
        std::cout << _s2.c_str() << std::endl;
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };

    void insert(size_t idx, size_t num, char ch) {
      _s1.insert(idx,num,ch);
      _s2.insert(idx,num,ch);
      if(_s2.size()!=_s1.size()) {
        WY_THROW( WyRet() );
      }
      if(_s2!=_s1.c_str()) {
        WY_THROW( WyRet() );
      }
    };
};

// [Syn] return a string from a table randomly picked
//
static const char* rand_str(void)
{
 static const char* SampStr[]={ 
    "","a","bb","ccc","dddd","eeeee","ffffff","ggggggg",
    "hhhhhhhh","iiiiiiiii","jjjjjjjjjj","kkkkkkkkkkk","llllllllllll"
    "mmmmmmmmmmmmm","nnnnnnnnnnnnnn","ooooooooooooooo","pppppppppppppppp",
    "qqqqqqqqqqqqqqqqq","rrrrrrrrrrrrrrrrrr","sssssssssssssssssss"
 };
 return SampStr[std::rand()%(sizeof(SampStr)/sizeof(char*))];
};

// [Syn] Check major operations with random length
//
static void t_rand_check(void)
try {
 const int LoopCount=1000;    // too large may cause CHK_STR failure (imp.)
 const size_t MaxRLen=10000;  // size for random source string, 10000 is enough

 CHK_STR ks1(rand_str());
 CHK_STR ks2(rand_str());

 for(int lcnt=0; lcnt<LoopCount; ++lcnt) {
   CHK_STR ks5(rand_str());
   CHK_STR ks6(ks5);
   CHK_STR ks7(std::rand()%MaxRLen,'x');
   ks6.reset(rand_str());
   switch((std::rand()%8)) {
     case 0: {
             CHK_STR str0;
             CHK_STR str1(rand_str());
             CHK_STR str2(rand_str());
             CHK_STR str3(std::rand()%MaxRLen,'x');
           }; break;
     case 1: {
            ks5.reset(rand_str());
            ks6.reset(WyStr(rand_str()));
            ks7.reset(std::rand()%MaxRLen,'q');
           }; break;
     case 2: {
            ks5.append('c');
            ks5.append(rand_str());
            ks6.append(WyStr(rand_str()));
            ks7.append(std::rand()%MaxRLen,'r');
           }; break;
     case 3: {
            ks5.append('c');
            ks5.append(ks6.c_str());
            ks6.append(ks5.wystr());
            ks7.append(std::rand()%MaxRLen,'s');
           }; break;
     case 4: {
            ks5.reset(ks6.c_str());
            ks6.reset(std::rand()%MaxRLen,'s');
            ks7.reset(std::rand()%MaxRLen,'t');
           }; break;
     case 5: {
            ks5.append(rand_str());
            ks6.append(WyStr(ks7.c_str())+ks5.c_str());
            ks7.insert((std::rand()%(ks7.size()+1)),"u");
           }; break;
     case 6: {
            ks5.insert((std::rand()%(ks5.size()+1)),rand_str());
            ks6.reset(rand_str());
            ks7.reset((std::rand()%(ks7.size()+1)),'v');
           }; break;
     default: {
            ks5.insert((std::rand()%(ks5.size()+1)),rand_str());
            ks6.insert((std::rand()%(ks6.size()+1)),rand_str());
            ks6.insert((std::rand()%(ks6.size()+1)),ks5.wystr());
            ks7.insert(std::rand()%(ks7.size()+1),std::rand()%MaxRLen,'t');
           }; break;
   };
 }
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << std::endl;
 throw;
}
catch(const std::exception& e) {
 std::cerr << e.what() << std::endl;
 throw;
}
catch(...) {
 WY_THROW( WyRet() );
};

static void t_mis(void)
try {
 // chk _toupper
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  if(&Wy::_toupper(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  CHK_DEFAULT(tmp);

  const char *StrUp="=1.2-* ABCDEFG+3Z	";
  const char *StrLo="=1.2-* abcdefg+3z	";

  tmp=StrLo;
  if(&Wy::_toupper(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=StrUp) {
    WY_THROW( WyRet() );
  }
  if(&Wy::_toupper(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=StrUp) {
    WY_THROW( WyRet() );
  }
 }

 // chk _tolower
 {
  WyStr tmp;
  CHK_DEFAULT(tmp);
  if(&Wy::_tolower(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  CHK_DEFAULT(tmp);

  const char *StrUp="=1.2-* ABCDEFG+3Z	";
  const char *StrLo="=1.2-* abcdefg+3z	";

  tmp=StrUp;
  if(&Wy::_tolower(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=StrLo) {
    WY_THROW( WyRet() );
  }
  if(&Wy::_tolower(tmp)!=&tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=StrLo) {
    WY_THROW( WyRet() );
  }
 }
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << std::endl;
 throw;
}
catch(const std::exception& e) {
 std::cerr << e.what() << std::endl;
 throw;
}
catch(...) {
 WY_THROW( WyRet() );
};

void ck_str(void)
{
 /* skip this check
 {
  WyStr tmp;
  WyRet r;
  if((r=tmp.reset(WyStr::max_capacity(),'a'))!=OK) {
   std::cout << "WyStr can not hold " << WyStr::max_capacity() << "characters\n";
  }
 }*/
 t1();
 t1_error(); 
 t2();
 t_share();
 t_mkstr();
 t_rand_check();
 t_mis();
};
