/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005
*/

#define WYLIB_SOURCE
#include "wy__cfunc.h"
#include "wydefs.h"   // for WY__CANCEL_POINT
#include <fcntl.h>

#ifdef WY_DEBUG
int wyc__fdcnt=0;
int wyc__fdcnt_max=0;
#endif

int wyc_open(const char* pathname, int flags)
{
 WY__CANCEL_POINT;
 const int v=::open(pathname,flags);
 WY__CANCEL_POINT;
#ifdef WY_DEBUG
 if(v!=-1) {
   ++wyc__fdcnt;
   if(wyc__fdcnt>wyc__fdcnt_max) {
     wyc__fdcnt_max=wyc__fdcnt;
   }
 }
#endif
 return(v);
};

int wyc_open(const char* pathname, int flags, mode_t mode)
{
 WY__CANCEL_POINT;
 const int v=::open(pathname,flags,mode);
 WY__CANCEL_POINT;
#ifdef WY_DEBUG
 if(v!=-1) {
   ++wyc__fdcnt;
   if(wyc__fdcnt>wyc__fdcnt_max) {
     wyc__fdcnt_max=wyc__fdcnt;
   }
 }
#endif
 return(v);
};

int wyc_creat(const char* pathname, mode_t mode)
{
 WY__CANCEL_POINT;
 const int v=::creat(pathname,mode);
 WY__CANCEL_POINT;
#ifdef WY_DEBUG
 if(v!=-1) {
   ++wyc__fdcnt;
   if(wyc__fdcnt>wyc__fdcnt_max) {
     wyc__fdcnt_max=wyc__fdcnt;
   }
 }
#endif
 return(v);
};

int wyc_dup(int oldfd)
{
 const int v=::dup(oldfd); 
#ifdef WY_DEBUG
 if(v!=-1) {
   ++wyc__fdcnt;
   if(wyc__fdcnt>wyc__fdcnt_max) {
     wyc__fdcnt_max=wyc__fdcnt;
   }
 }
#endif
 return(v);
};

// [Syn] Is the given fd an open file descriptor
//
// [Ret] 1= fd is an open file descriptor
//       0= otherwise
//
int wyc__is_open(int fd)
{
 return (::fcntl(fd,F_GETFL)==-1)? 0:1;
};

int wyc_dup2(int oldfd, int newfd)
{
#ifdef WY_DEBUG
 int t= wyc__is_open(newfd);
 const int v=::dup2(oldfd,newfd); 
 if(v!=-1) {
   ++wyc__fdcnt;
   wyc__fdcnt-=t;
   if(wyc__fdcnt>wyc__fdcnt_max) {
     wyc__fdcnt_max=wyc__fdcnt;
   }
 }
 return(v);
#else
 return ::dup2(oldfd,newfd); 
#endif
};

int wyc_fstat(int fd, struct stat* buf)
{ return ::fstat(fd,buf); };

int wyc_stat(const char* pathname, struct stat* buf)
{ return ::stat(pathname,buf); };

int wyc_fsync(int fd)
{
 WY__CANCEL_POINT;
 const int v( ::fsync(fd) );
 WY__CANCEL_POINT;
 return(v);
};

int wyc_fdatasync(int fd)
{ return ::fdatasync(fd); };

int wyc_isatty(int fd)
{
 return(::isatty(fd)); 
};
