/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   Utilities for library/program development

   Note: Support single arithmetic type (except char, it seemd have to)
         For arithmetic type t, f<< t, and f<< Wy::wrd(t) are both allowed
*/

#ifndef WY_UTY_H__
#define WY_UTY_H__
#define WY_UTY_VERSION 30

#include "wystr.h"
#include "wynum.h"
#include "wybyteflow.h"

namespace Wy {
  // [Evaluating]
  WyRet gethostname(WyStr& buf);

//  [Internal] Write blen bytes pointed by buf to the device.
//
//  Note: info. of the number of written characters is lost
//
//  [\fBRet\fP] OK
//     Wym_EBADF       Object is not valid for the operation
//     Wym_EINVAL      Object attatched device is not suitable
//     Wym_EFAULT      buf not accessible
//     Wym_EFBIG       Attempt to write beyond the device limit 
//     Wym_EPIPE       Write to a pipe or FIFO with no reader
//     Wym_EAGAIN      Non-blocking I/O (write would block)
//     Wym_EINTR       Interrupted by a signal before data written
//     Wym_ENOSPC      Device left no space for the operation
//     Wym_EIO         I/O error
//
WyRet wy__bwrite(WyByteFlow& bf, const char* buf, size_t blen);

};

// [Syn] Operator<< overload
//       Write bytes indicated by the right-hand side argument to the
//       left-hand side WyByteFlow object.
// 
WyByteFlow& operator <<(WyByteFlow& bf, const WyStr& str);
WyByteFlow& operator <<(WyByteFlow& bf, WyCSeg str);
WyByteFlow& operator <<(WyByteFlow& bf, const char* cstr);
WyByteFlow& operator <<(WyByteFlow& bf, char* cstr);
WyByteFlow& operator <<(WyByteFlow& bf, char ch);
WyByteFlow& operator <<(WyByteFlow& bf, const void* p);

// [Syn] Operator<< overload
//       Convert the right-hand side arithmetic type argument to radix 10 
//       string representation and write to the left-hand side WyByteFlow
//       object.
// 
WyByteFlow& operator <<(WyByteFlow&, signed char num);
WyByteFlow& operator <<(WyByteFlow&, signed short num);
WyByteFlow& operator <<(WyByteFlow&, signed int num);
WyByteFlow& operator <<(WyByteFlow&, signed long num);
WyByteFlow& operator <<(WyByteFlow&, signed long long num);
WyByteFlow& operator <<(WyByteFlow&, unsigned char num);
WyByteFlow& operator <<(WyByteFlow&, unsigned short num);
WyByteFlow& operator <<(WyByteFlow&, unsigned int num);
WyByteFlow& operator <<(WyByteFlow&, unsigned long num);
WyByteFlow& operator <<(WyByteFlow&, unsigned long long num);
WyByteFlow& operator <<(WyByteFlow&, float);
WyByteFlow& operator <<(WyByteFlow&, double);

//WyByteFlow& operator <<(WyByteFlow&, const void* p);

// [Syn] Operator>> overload
//       Read data from the left-hand side WyByteFlow object and store into
//       the right-hand side object
//
WyByteFlow& operator >>(WyByteFlow& bf, WyStr& str);
WyByteFlow& operator >>(WyByteFlow& bf, char& ch);
inline WyByteFlow& operator >>(WyByteFlow& bf, unsigned char& ch)
     { return operator>>(bf,(char&)ch); };
inline WyByteFlow& operator >>(WyByteFlow& bf, signed char& ch)
     { return operator>>(bf,(char&)ch); };
#endif

#include "wy__rdque.h"      // included in wy_uty.h for now
