/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Documentation is in file wybyteflow.3wy
*/
#ifndef WYBYTEFLOW_H__
#define WYBYTEFLOW_H__
#define WYBYTEFLOW_VERSION 30

#include "wysysfile.h"

class WyByteFlow : public WySysFile {
    static const Wy__CIF BfCif;

    // Hidden
    const WyByteFlow & operator =(const WyByteFlow &);
    bool operator==(const WyByteFlow&) const;
    bool operator!=(const WyByteFlow&) const;
    
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    explicit  // no implicit conversion
    WyByteFlow(const Wy__CIF& cif=BfCif);
    WyByteFlow(WyFileHandle fh, const Wy__CIF& cif=BfCif);
    WyByteFlow(const WySysFile& sysf, const Wy__CIF& cif=BfCif);

    // [Added/forward]
    WyByteFlow(const char* pathname,int f, const Wy__CIF& cif=BfCif);
    WyByteFlow(const WyStr& pathname,int f, const Wy__CIF& cif=BfCif);

    // Note: If not provide copy construct, WyByteFlow(WyByteFlow&) throws
    //       WySysFile::Reply (fixed by provides one, so may guess that
    //       compiler generated its own copy)
    //
    WyByteFlow(const WyByteFlow& sysf, const Wy__CIF& cif=BfCif);

    // [Move Constructor]
    //
    // Note: sysf must be whole object except in initialization list
    //
    WyByteFlow(WyByteFlow& sysf, Wy::ByMove_t) WY__TSPC();

    //virtual ~WyByteFlow();
 
    // [Derived]
    //bool is_default(void) const WY__TSPC();
    //WyFileHandle fh(void) const WY__TSPC();
    //mode_t ftype(void) const WY__TSPC();
    //WyRet stat(WyFileStat& filestat) const;

    // [Override/forward]
    virtual WyRet reset(void)
            { return WySysFile::reset(); };
    virtual WyRet reset(WyFileHandle h)
            { return WySysFile::reset(h); };
    virtual WyRet reset(const WySysFile& sysf)
            { return WySysFile::reset(sysf); };
    virtual WyRet reset(const char* pathname, int flags)
            { return WySysFile::reset(pathname,flags); };
    virtual WyRet reset(const WyStr& pathname, int flags)
            { return WySysFile::reset(pathname,flags); };

    // [Derived]
    //virtual WyRet _swap(WySysFile& bf) WY__TSPC();

    // [Override]
    virtual WyByteFlow* _alloc(WyRet&) const;

    // [Added]
    virtual WyRet read(void*, size_t, size_t&);
    virtual WyRet read(WyStr&, size_t, size_t&);

    // [Added]
    //
    // Note: This member probably just works fine for stream-like device
    //       (stream socket, regular file), but many actually programs use
    //       this function.
    //
    virtual WyRet _xread(void*, size_t, size_t&);

    virtual WyRet write(const void*, size_t, size_t&);
    virtual WyRet write(WyCSeg, size_t&);
    virtual WyRet write(const WyStr&, size_t&);

    WyRet _fsync(void);
    WyRet _fdatasync(void);
    WyRet _tcdrain(void);
    bool  _isatty(void) const;

    bool wy_is_std_fd(void) const WY__TSPC();
};

namespace Wy {
 extern WyByteFlow& cerr;
 extern WyByteFlow& cin;
 extern WyByteFlow& cout;
};

#endif
