/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/
#define WYLIB_SOURCE
#include "wychrfile.h"
#include <memory>   // for new
#include <typeinfo>

const char WyChrFile::class_name[]="WyChrFile";

const Wy__CIF WyChrFile::ChrCif( Wy__FTSet(S_IFCHR) );

WyChrFile::WyChrFile()
try : WyByteFlow(ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(WyFileHandle fh)
try : WyByteFlow(fh,ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(const WySysFile& chrf)
try : WyByteFlow(chrf,ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(const char* pathname,int f)
try : WyByteFlow(pathname,f,ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(const WyStr& pathname,int f)
try : WyByteFlow(pathname,f,ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(const WyChrFile &chrf)
try : WyByteFlow(chrf,ChrCif) {
}
catch(const WyByteFlow::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WY_THROW( WyRet(e) );
};

WyChrFile::WyChrFile(WyChrFile& chrf, Wy::ByMove_t) WY__TSPC()
 : WyByteFlow(chrf,Wy::ByMove)
{
};

WyChrFile* WyChrFile::_alloc(WyRet& r) const
try {
 if(WY__EQU_TYPEID(*this,WyChrFile)==false) {
   r=Wym_ENOSYS; WY_HERE(r);        // _alloc not overridden
   return(NULL);
 }
 WyChrFile* p=new(std::nothrow) WyChrFile();
 if(p==NULL) {
   r=Wym_ENOMEM; WY_HERE(r);
 } else {
   r=OK;
 }
 return(p);
}
catch(const Reply& e) {
 r=e; WY_HERE(r);
 return(NULL);
}
catch(const WyRet& e) {
 WY_THROW( WyRet(e) );
};
