/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WYDIRENT_H__
#define WYDIRENT_H__
#define WYDIRENT_VERSION 30

#include "wyret.h"
#include <sys/types.h>
#include <dirent.h>

/*
   WyDirEnt defines the data item that Wy_DirFile reads.
*/
class WyDirEnt {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyDirEnt() WY__TSPC();
    WyDirEnt(const WyDirEnt &src) WY__TSPC();
    WyDirEnt(WyDirEnt &src, Wy::ByMove_t) WY__TSPC();
    WyDirEnt(const struct dirent& dt) WY__TSPC();

    bool is_default(void) const WY__TSPC();

    long _ino(void) const WY__TSPC();
    off_t _off(void) const WY__TSPC();
    size_t _reclen(void) const WY__TSPC();
    const char* filename(void) const WY__TSPC();
    
    void reset(void) WY__TSPC();
    void reset(const WyDirEnt &src) WY__TSPC();
    void swap(WyDirEnt& src) WY__TSPC();
    const WyDirEnt& operator =(const WyDirEnt &rhs) WY__TSPC();

    bool operator ==(const WyDirEnt& rhs) const WY__TSPC();
    bool operator !=(const WyDirEnt& rhs) const WY__TSPC();

    // [Internal] Get ::dirent pointer of this object
    //
    struct ::dirent* wy_dent_ptr(void) WY__NOTHROW__;
  private:
    struct ::dirent _ent;
};

/*
// Wy_Array specilization to support struct dirent
//
#include "wy_array.h"
template<>
struct Wy_Array< struct dirent > : Wy__ArrayValue< struct dirent > {
  Wy_Array() : Wy__ArrayValue< struct dirent >() {};
  Wy_Array(size_t s,const struct dirent& elem) : Wy__ArrayValue< struct dirent >(s,elem) {};
  Wy_Array(const Wy_Array& s) : Wy__ArrayValue< struct dirent >(s) {};
  Wy_Array(Wy_Array& p, Wy::ByMove_t t) : Wy__ArrayValue< struct dirent >(p,t) {};
};
*/

#endif
