/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2004
*/
#define WYLIB_SOURCE
#include "wystr.h"
#include "wy__hstr.h"
#include <limits>

const char WyStr::class_name[]="WyStr";

WyStr::size_type WyStr::max_capacity(void) WY__TSPC()
{
 return Wy__HStr::max_capacity(); 
};

WyStr::size_type WyStr::min_capacity(void) WY__TSPC()
{
 return Wy__HStr::min_capacity();
};

class WyStr::_Assertion {
  public:
    _Assertion() {
       if(WyStr::max_capacity()>static_cast<WyStr::size_type>(
                           (std::numeric_limits<ssize_t>::max()/2-1))) {
         WY_TERMINATE("");
       };
       if(NULL!=(void*)0) {
         WY_TERMINATE("");
       };
    };
} static const wy__do_not_use_me__;

WyStr::WyStr() WY__TSPC(Reply)
try : _imp(Wy__HStr::alloc())
{
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW(Reply(e));
};

WyStr::WyStr(const WyStr& str) WY__TSPC(Reply)
try : _imp(Wy__HStr::alloc(*str._imp))
{
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW(Reply(e));
};

WyStr::WyStr(WyStr& str, Wy::ByMove_t) WY__TSPC()
 : _imp(str._imp)
{
 str._imp=NULL;
};

WyStr::WyStr(const WyCSeg& cs) WY__TSPC(Reply)
try : _imp(Wy__HStr::alloc(cs))
{
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW(Reply(e));
};

WyStr::WyStr(size_type num, WyStr::value_type ch) WY__TSPC(Reply)
try : _imp(Wy__HStr::alloc(num,ch))
{
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW(Reply(e));
};

WyStr::WyStr(Wy__HStr* hstr) WY__TSPC()
 : _imp(hstr) {
};
//------------------------
WyStr::~WyStr() WY__TSPC()
{
 Wy__HStr::free(_imp); _imp=NULL;
};

bool WyStr::is_default(void) const WY__TSPC()
{
 return(_imp->size()==0);
};

void WyStr::reset(void) WY__TSPC()
{
 _imp->set_string();
};

WyRet WyStr::reset(const WyStr& str) WY__TSPC()
{
 const WyRet r(_imp->set_string(*str._imp));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(*str._imp); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::reset(const WyCSeg& cs) WY__TSPC()
{
 const WyRet r(_imp->set_string(cs));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(cs); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::reset(size_type cnum, WyStr::value_type ch) WY__TSPC()
{
 const WyRet r(_imp->set_string(cnum,ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(cnum,ch); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

//
WyStr::size_type WyStr::size(void) const WY__TSPC()
{
 return _imp->size();
};

const WyStr::value_type* WyStr::begin(void) const WY__TSPC()
{
 return _imp->_begin();
};

const WyStr::value_type* WyStr::end(void) const WY__TSPC()
{
 return _imp->_end();
};

const WyStr::value_type& WyStr::operator [](size_type pos) const WY__TSPC()
{
 return _imp->_at(pos);
};

WyStr::value_type& WyStr::operator [](size_type pos) WY__TSPC()
{
 return _imp->_at(pos);
};

const WyStr::value_type& WyStr::at(size_type pos) const WY__TSPC(Reply)
{
 if(pos>=_imp->size()) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return _imp->_at(pos);
};

WyStr::value_type& WyStr::at(size_type pos) WY__TSPC(Reply)
{
 if(pos>=_imp->size()) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return _imp->_at(pos);
};

const WyStr::value_type* WyStr::data(void) const WY__TSPC()
{
 return _imp->data();
};

const WyStr::value_type* WyStr::c_str(void) const WY__TSPC()
{
 return _imp->c_str();
};

bool WyStr::operator ==(const WyStr& rhs) const WY__TSPC()
{
 return Wy::_strcmp(this->cseg(),rhs._imp->cseg())==0;
};

bool WyStr::operator !=(const WyStr& rhs) const WY__TSPC()
{
 return Wy::_strcmp(this->cseg(),rhs._imp->cseg())!=0;
};

bool WyStr::operator ==(const WyCSeg& rhs) const WY__TSPC()
{
 return Wy::_strcmp(this->cseg(),rhs)==0;
};

bool WyStr::operator !=(const WyCSeg& rhs) const WY__TSPC()
{
 return Wy::_strcmp(this->cseg(),rhs)!=0;
};

bool WyStr::operator ==(const char* rhs) const WY__TSPC(WyCSeg::Reply)
{
 return Wy::_strcmp(this->cseg(),WyCSeg(rhs))==0;
};

bool WyStr::operator !=(const char* rhs) const WY__TSPC(WyCSeg::Reply)
{
 return Wy::_strcmp(this->cseg(),WyCSeg(rhs))!=0;
};

WyRet WyStr::append(const WyStr& str) WY__TSPC()
{
 const WyRet r(_imp->append(*str._imp));
 if(r==OK) {
   return(OK);
 }

 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }
 // realloc only for EFBIG
 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),str._imp->cseg()); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::append(value_type ch) WY__TSPC()
{
 const WyRet r(_imp->append(ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }
 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),ch); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::append(WyStr::size_type cnum,WyStr::value_type ch) WY__TSPC()
{
 const WyRet r(_imp->append(cnum,ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),cnum,ch); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::append(const WyCSeg& cs) WY__TSPC()
{
 const WyRet r(_imp->append(cs));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),cs); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

const WyStr& WyStr::operator +=(const WyStr& str) WY__TSPC(Reply)
{ 
 const WyRet r(_imp->append(str.cseg()));
 if(r==OK) {
   return(*this);
 }
 if(r!=Wym_EFBIG) {
   WY_THROW( Reply(r) );
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),str.cseg()); }
 catch(const Wy__HStr::Reply& e) {
   WY_THROW( Reply(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(*this);
};

const WyStr& WyStr::operator +=(const WyCSeg& cs) WY__TSPC(Reply)
{
 const WyRet r(_imp->append(cs));
 if(r==OK) {
   return(*this);
 }
 if(r!=Wym_EFBIG) {
   WY_THROW( Reply(r) );
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),cs); }
 catch(const Wy__HStr::Reply& e) {
   WY_THROW( Reply(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(*this);
};

const WyStr& WyStr::operator +=(value_type ch) WY__TSPC(Reply)
{
 const WyRet r(_imp->append(ch));
 if(r==OK) {
   return(*this);
 }
 if(r!=Wym_EFBIG) {
   WY_THROW( Reply(r) );
 }
 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(),ch); }
 catch(const Wy__HStr::Reply& e) {
   WY_THROW( Reply(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(*this);
};

WyCSeg WyStr::cseg(void) const WY__TSPC()
{
 return _imp->cseg();
};

WyCSeg WyStr::cseg(size_type idx) const WY__TSPC(Reply)
try {
 return _imp->cseg(idx);
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW( Reply(e) );
};

WyCSeg WyStr::cseg(size_type idx, size_type num) const WY__TSPC(Reply)
try {
 return _imp->cseg(idx,num);
}
catch(const Wy__HStr::Reply& e) {
 WY_THROW( Reply(e) );
};

bool WyStr::find(size_type* res,value_type ch) const WY__TSPC()
{
 return _imp->find(res,ch);
};

bool WyStr::find(size_type* res,const WyCSeg& cseg) const WY__TSPC()
{
 return _imp->find(res,0,cseg);
};

bool WyStr::find(size_type* res,size_type idx,value_type ch) const WY__TSPC()
{
 return _imp->find(res,idx,ch);
};

bool WyStr::find(size_type* res,size_type idx,const WyCSeg& cseg) const WY__TSPC()
{
 return _imp->find(res,idx,cseg);
};

bool WyStr::rfind(size_type* res,value_type ch) const WY__TSPC()
{
 return _imp->rfind(res,ch);
};

bool WyStr::rfind(size_type* res,const WyCSeg& cseg) const WY__TSPC()
{
 return _imp->rfind(res,cseg);
};

bool WyStr::rfind(size_type* res,size_type idx,value_type ch) const WY__TSPC()
{
 return _imp->rfind(res,idx,ch);
};

bool WyStr::rfind(size_type* res,size_type idx,const WyCSeg& cseg) const WY__TSPC()
{
 return _imp->rfind(res,idx,cseg);
};

bool WyStr::find_not(size_type* res,size_type idx,value_type ch) const WY__TSPC()
{
 return _imp->find_not(res,idx,ch);
};

bool WyStr::rfind_not(size_type* res,size_type idx,value_type ch) const WY__TSPC()
{
 return _imp->rfind_not(res,idx,ch);
};

const WyStr& WyStr::operator =(const WyStr& rhs) WY__TSPC(Reply)
{
 const WyRet r(_imp->set_string(*rhs._imp));
 if(r==OK) {
   return(*this);
 }
 if(r!=Wym_EFBIG) {
   WY_THROW( Reply(r) );
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(*rhs._imp); }
 catch(const Wy__HStr::Reply& e) {
   WY_THROW( Reply(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(*this);
};

const WyStr& WyStr::operator =(const WyCSeg& rhs) WY__TSPC(Reply)
{
 const WyRet r(_imp->set_string(rhs));
 if(r==OK) {
   return(*this);
 }
 if(r!=Wym_EFBIG) {
   WY_THROW( Reply(r) );
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(rhs); }
 catch(const Wy__HStr::Reply& e) {
   WY_THROW( Reply(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(*this);
};

void WyStr::swap(WyStr& str) WY__TSPC()
{
 Wy__Base::vswap(_imp,str._imp);
};

WyRet WyStr::insert(size_type idx,const WyStr& str) WY__TSPC()
{
 const WyRet r(_imp->insert(idx,*str._imp));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),str._imp->cseg(),_imp->cseg(idx)); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::insert(size_type idx,const WyCSeg& cseg) WY__TSPC()
{
 const WyRet r(_imp->insert(idx,cseg));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),cseg,_imp->cseg(idx)); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::insert(size_type idx,size_type cnum,value_type ch) WY__TSPC()
{
 const WyRet r(_imp->insert(idx,cnum,ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),cnum,ch,_imp->cseg(idx)); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::erase(size_type idx) WY__TSPC()
{
 WY_RETURN( _imp->erase(idx) );
};

WyRet WyStr::erase(size_type idx,size_type len) WY__TSPC()
{
 WY_RETURN( _imp->erase(idx,len) );
};

WyRet WyStr::resize(size_type num) WY__TSPC()
{
 // Note: Asserted no error if num<_capacity()
 const WyRet r(_imp->resize(num));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { 
   hptr= Wy__HStr::alloc(_imp->cseg(),num-_imp->size(),value_type(0));
 }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::resize(size_type num,value_type ch) WY__TSPC()
{
 // Note: Asserted no error if num<_capacity()
 const WyRet r(_imp->resize(num,ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { 
   hptr= Wy__HStr::alloc(_imp->cseg(),num-_imp->size(),ch);
 }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::replace(size_type idx,size_type num,const WyStr& str) WY__TSPC()
{
 const WyRet r(_imp->replace(idx,num,*str._imp));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),str.cseg(),
                             _imp->cseg(idx+_imp->cseg(idx,num).size()));
 }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::replace(size_type idx,size_type num,const WyCSeg& cseg) WY__TSPC()
{
 const WyRet r(_imp->replace(idx,num,cseg));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),cseg,
                             _imp->cseg(idx+_imp->cseg(idx,num).size()));
 }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::replace(size_type idx,size_type num,size_type cnum,value_type ch) WY__TSPC()
{
 const WyRet r(_imp->replace(idx,num,cnum,ch));
 if(r==OK) {
   return(OK);
 }
 if(r!=Wym_EFBIG) {
   WY_RETURN(r);
 }

 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(_imp->cseg(0,idx),cnum,ch,
                             _imp->cseg(idx+_imp->cseg(idx,num).size()));
 }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

int WyStr::strcmp(const WyStr& str) const WY__TSPC()
{
 return _imp->strcmp(*str._imp);
};

int WyStr::strcmp(const WyCSeg& cseg) const WY__TSPC()
{
 return _imp->strcmp(cseg);
};

int WyStr::strcmp(const char* cstr) const WY__TSPC(WyCSeg::Reply)
{
 return _imp->strcmp( WyCSeg(cstr) );
};

WyStr::size_type WyStr::_capacity(void) const WY__TSPC()
{
 return(_imp->_capacity());
};

WyRet WyStr::copyto(size_type* rlen,value_type* buf,size_type blen,size_type idx) const WY__TSPC()
{
 WY_RETURN( _imp->copyto(rlen,buf,blen,idx) );
};

WyRet WyStr::_reserve(size_type num) WY__TSPC()
{
 if(num<=_capacity()) {
   return(OK);
 }
 Wy__HStr* hptr(NULL);
 try { hptr= Wy__HStr::alloc(num); }
 catch(const Wy__HStr::Reply& e) {
   WY_RETURN( WyRet(e) );
 };
 const WyRet r( hptr->set_string(*_imp) );
 if(r!=OK) {
   // for now, this version of set_string only fails on Wym_ELOOP
   // it should always succeed.
   Wy__HStr::free(hptr);
   #ifdef WY_DEBUG
     WY_TERMINATE("");
   #else
     WY_RETURN(Wym_ENOMEM);   // Hide bug as normal failure
   #endif
 }
 Wy__HStr::free(_imp);
 _imp=hptr;
 return(OK);
};

WyRet WyStr::_setsize(size_t nsize) WY__TSPC()
{
 return _imp->_setsize(nsize);
};

WyStr operator +(const WyStr& lhs,const WyStr& rhs) WY__TSPC(WyStr::Reply)
try {
 return WyStr( Wy__HStr::alloc(lhs.cseg(),rhs.cseg()) );
}
catch(const Wy__HStr::Reply& e) {
  WY_THROW( WyStr::Reply(e) );
};

WyStr operator +(const WyStr& lhs,const WyCSeg& rhs) WY__TSPC(WyStr::Reply)
try {
  return WyStr( Wy__HStr::alloc(lhs.cseg(),rhs) );
}
catch(const Wy__HStr::Reply& e) {
  WY_THROW( WyStr::Reply(e) );
};

WyStr operator +(const WyStr& str,WyStr::value_type ch) WY__TSPC(WyStr::Reply)
try {
 return WyStr( Wy__HStr::alloc(str.cseg(),ch) );
}
catch(const Wy__HStr::Reply& e) {
  WY_THROW( WyStr::Reply(e) );
};

WyStr operator +(const WyCSeg& cs,const WyStr& str) WY__TSPC(WyStr::Reply)
try {
 return WyStr( Wy__HStr::alloc(cs,str.cseg()) );
}
catch(const Wy__HStr::Reply& e) {
  WY_THROW( WyStr::Reply(e) );
};

WyStr operator +(WyStr::value_type ch,const WyStr& str) WY__TSPC(WyStr::Reply)
try {
 return WyStr( Wy__HStr::alloc(ch,str.cseg()) );
}
catch(const Wy__HStr::Reply& e) {
  WY_THROW( WyStr::Reply(e) );
};

//----------------------------------------------------------------

//
// [Syn] Make string representation of the given argument file_name/line_num.
//
// [Ret] WyStr, string representation of the arguments
//              in the form: file_name '(' line_num ')'
//
typedef int wy__line_num_type;
static WyStr mkstr_sli(const char *file_name,wy__line_num_type line_num) WY__TSPC(WyStr::Reply)
try {
 WyStr nstr;

 if(file_name!=NULL) {
#ifndef WY_SHORT_SLI    // personal use for my application
   nstr=file_name;
#else
  //
  // Sample the first MaxFlen characters of file_name or until the dot 
  // character as the source file name information.
  //
  const int MaxFlen=8;  
  int flen;
  for(flen=0; flen<=MaxFlen; ++flen) {
     const char ch=file_name[flen];
     if((ch==0)||(ch=='.')) {
       nstr.reset(WyCSeg(file_name,flen));
       break;
     }
  }
#endif
 };

#ifndef WY_SHORT_SLI
 nstr+='(';
#endif
 
 //
 // Convert line_num number
 // 
 char dbuf[2+3*sizeof(wy__line_num_type)];
 std::div_t dt;
 int n;

 if(line_num<0) {
   dt.quot=-line_num;
   nstr+='-';
 } else {
   dt.quot=line_num;
 }
  
 n=0;
 do {
   dt=std::div(dt.quot,10);
   dbuf[n++]=(char)(dt.rem);
 } while(dt.quot>0);

 for(--n; n>=0; --n) {
   const char ch=dbuf[n];
   nstr+= (ch>9)?  char(ch+'7'):char(ch+'0');
 }
#ifndef WY_SHORT_SLI
 nstr+=')';
#endif
 return(nstr);
}
catch(const WyStr::Reply&) {
 throw;
};

//---------------------------------------------------------------

WyStr& Wy::_toupper(WyStr& str) WY__TSPC()
{
 const WyStr::value_type *pend( str.data()+str.size() );
 for(WyStr::value_type *p=&str[0]; p!=pend; ++p) {
   if((*p>='a')&&(*p<='z')) {
     *p-=WyStr::value_type(0x20);
   }
 }
 return str;
};

WyStr& Wy::_tolower(WyStr& str) WY__TSPC()
{
 const WyStr::value_type *pend( str.data()+str.size() );
 for(WyStr::value_type *p=&str[0]; p!=pend; ++p) {
   if((*p>='A')&&(*p<='Z')) {
     *p+=WyStr::value_type(0x20);
   }
 }
 return str;
};

WyStr Wy::wrd(const WyReply& em) WY__TSPC(WyStr::Reply)
try {
 WyRet r;
 WyStr str;
 if((r=str._reserve( em->get_reply(NULL,0) ))!=OK) {
   WY_THROW( WyStr::Reply(r) );
 }
 if((r=str._setsize( em->get_reply(&str[0],str._capacity()) ))!=OK) {
   WY_THROW(r);
 }
 return(str);
}
catch(const WyStr::Reply& e) {
 if(e==Wym_ENOMEM) {
   throw;
 }
 WY_TERMINATE("");
};
 
WyStr Wy::wrd(const Wy_SrcLoc& s) WY__TSPC(WyStr::Reply)
try {
 return mkstr_sli(s.src_file(),s.src_line());
}
catch(const WyStr::Reply& e) {
 if(e==Wym_ENOMEM) {
   throw;
 }
 WY_TERMINATE("");
};

WyStr Wy::wrd(const WyRet& arg) WY__TSPC(WyStr::Reply)
try {
#ifndef WYLIB_CONFIG_NORSLI
 WyRet r;
 WyStr str;

 if((r=str._reserve( arg->get_reply(NULL,0) ))!=OK) {
   WY_THROW( WyStr::Reply(r) );
 }
 if((r=str._setsize( arg->get_reply(&str[0],str._capacity()) ))!=OK) {
   WY_THROW(r);
 }
 if(arg.sli_ptr()!=NULL) {
   str+=", ";
   str+=Wy::wrd(*arg.sli_ptr());
 }
 return(str);
#else
 return Wy::wrd( static_cast<WyReply>(arg) );
#endif
}
catch(const WyStr::Reply& e) {
  throw;
};

//---------------------------------------------------------------
