/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/
#ifndef WYTERMIOS_H__
#define WYTERMIOS_H__
#define WYTERMIOS_VERSION 30

#include "wyret.h"
#include <termios.h>

/*
  WyTermios defines the control attribute of WyTerminal (struct termios)
*/
class WyTermios {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyTermios() WY__TSPC();

    WyTermios(const WyTermios& tio) WY__TSPC();
    WyTermios(WyTermios& tio, Wy::ByMove_t) WY__TSPC();

    bool is_default(void) const WY__TSPC();

    void reset(void) WY__TSPC();
    void reset(const WyTermios& tio) WY__TSPC();

    void swap(WyTermios& src) WY__TSPC();

    const WyTermios& operator =(const WyTermios &tio) WY__TSPC();

    speed_t getispeed(void) const WY__TSPC();

    speed_t getospeed(void) const WY__TSPC();

    WyRet setispeed(speed_t speed) WY__TSPC();

    WyRet setospeed(speed_t speed) WY__TSPC();
    
    tcflag_t iflag(void) const WY__TSPC();
    void iflag(tcflag_t v) WY__TSPC();
    void iflag_or(tcflag_t v) WY__TSPC();
    void iflag_and(tcflag_t v) WY__TSPC();

    tcflag_t oflag(void) const WY__TSPC();
    void oflag(tcflag_t v) WY__TSPC();
    void oflag_or(tcflag_t v) WY__TSPC();
    void oflag_and(tcflag_t v) WY__TSPC();

    tcflag_t cflag(void) const WY__TSPC();
    void cflag(tcflag_t v) WY__TSPC();
    void cflag_or(tcflag_t v) WY__TSPC();
    void cflag_and(tcflag_t v) WY__TSPC();

    tcflag_t lflag(void) const WY__TSPC();
    void lflag(tcflag_t v) WY__TSPC();
    void lflag_or(tcflag_t v) WY__TSPC();
    void lflag_and(tcflag_t v) WY__TSPC();

    cc_t cc_VSTOP(void) const WY__TSPC();
    void cc_VSTOP(cc_t v) WY__TSPC();

    cc_t cc_VSTART(void) const WY__TSPC();
    void cc_VSTART(cc_t v) WY__TSPC();

    cc_t cc_VSUSP(void) const WY__TSPC();
    void cc_VSUSP(cc_t v) WY__TSPC();

    cc_t cc_VQUIT(void) const WY__TSPC();
    void cc_VQUIT(cc_t v) WY__TSPC();

    cc_t cc_VKILL(void) const WY__TSPC();
    void cc_VKILL(cc_t v) WY__TSPC();

    cc_t cc_VERASE(void) const WY__TSPC();
    void cc_VERASE(cc_t v) WY__TSPC();

    cc_t cc_VEOL(void) const WY__TSPC();
    void cc_VEOL(cc_t v) WY__TSPC();

    cc_t cc_VEOF(void) const WY__TSPC();
    void cc_VEOF(cc_t v) WY__TSPC();

    cc_t cc_VINTR(void) const WY__TSPC();
    void cc_VINTR(cc_t v) WY__TSPC();

    cc_t cc_VTIME(void) const WY__TSPC();
    void cc_VTIME(cc_t v) WY__TSPC();

    cc_t cc_VMIN(void) const WY__TSPC();
    void cc_VMIN(cc_t v) WY__TSPC();

    bool operator ==(const WyTermios& rhs) const WY__TSPC();
    bool operator !=(const WyTermios& rhs) const WY__TSPC()
                    { return !operator==(rhs); };

    // [Internal] Get ::termios pointer of this object
    //
    struct ::termios* wy_tio_ptr(void) WY__NOTHROW__;
    const struct ::termios* wy_tio_ptr(void) const WY__NOTHROW__;
  private:
    struct ::termios _tio;
};

/*
// Wy_Array specilization to support struct ::termios
//
#include "wy_array.h"
template<>
struct Wy_Array< struct ::termios > : Wy__ArrayValue< struct ::termios > {
  Wy_Array() : Wy__ArrayValue< struct ::termios >() {};
  Wy_Array(size_t s,const struct ::termios& elem) : Wy__ArrayValue< struct ::termios >(s,elem) {};
  Wy_Array(const Wy_Array& s) : Wy__ArrayValue< struct ::termios >(s) {};
  Wy_Array(Wy_Array& p, Wy::ByMove_t t) : Wy__ArrayValue< struct ::termios >(p,t) {};
};
*/

#endif
